

# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/secure/services/account.proto, blickfeld/secure/services/application_key.proto, blickfeld/secure/services/authentication.proto, blickfeld/secure/services/device_credentials.proto, blickfeld/secure/services/firewall.proto, blickfeld/secure/services/health.proto, blickfeld/secure/services/license.proto, blickfeld/secure/services/session.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from .. import (
    config as _config__,
    data as _data__,
)


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class ApplicationKeyListRequest(betterproto.Message):
    """Request to list applicaion keys"""

    account_uuid: str = betterproto.string_field(1)
    """
    Optional uuid of the account for which to list the application keys. If not
    provided the uuid from the presented JWT token will be used.
    """


@dataclass(eq=False, repr=False)
class ApplicationKeyListResponse(betterproto.Message):
    """Response for listing existing application keys."""

    application_key: Dict[str, "_config__.ApplicationKey"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """
    Application key mapping from application-key-uuid (key) to application key
    details (value).
    """


@dataclass(eq=False, repr=False)
class ApplicationKeyCreateRequest(betterproto.Message):
    """Request to create an application key."""

    account_uuid: str = betterproto.string_field(1)
    """
    Optional uuid of the account to which the application key is associated. If
    not provided the uuid from the presented JWT token will be used.
    """

    application_key: "_config__.ApplicationKey" = betterproto.message_field(2)
    """
    The application key to create. Minimum (optional) input is the value set in
    the `purpose` field. Everything else will be generated and added on the
    server side.
    """


@dataclass(eq=False, repr=False)
class ApplicationKeyCreateResponse(betterproto.Message):
    """
    Response containing the plain-text key information of the generated
    application key.   [NOTE] ==== The generated key in plain text will be
    returned only once. ====
    """

    uuid: str = betterproto.string_field(2)
    """The uuid of the application key."""

    key: str = betterproto.string_field(1)
    """
    The generated key in plain text. This key has to be used whenever an
    application needs a token for authenticated access.
    """


@dataclass(eq=False, repr=False)
class ApplicationKeyDeleteRequest(betterproto.Message):
    """Request to delete an application key."""

    uuid: str = betterproto.string_field(1)
    """
    The uuid of the application key to delete. Once the application key has
    been deleted all future authentication attempts with this application key
    will be blocked.
    """


@dataclass(eq=False, repr=False)
class AccountListResponse(betterproto.Message):
    """Response for list of accounts"""

    account: Dict[str, "_config__.Account"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    """Accounts mapping from account-uuid (key) to account details (value)."""


@dataclass(eq=False, repr=False)
class AccountCreateRequest(betterproto.Message):
    """Request to create a new user"""

    account: "_config__.Account" = betterproto.message_field(1)
    """the account to create"""


@dataclass(eq=False, repr=False)
class AccountCreateResponse(betterproto.Message):
    """
    Response after account has been created.   [NOTE] ==== The generated
    activation password in plain text will be returned only once. ====
    """

    activation_password: str = betterproto.string_field(1)
    """the activation password in plain text"""

    uuid: str = betterproto.string_field(2)
    """the uuid of the account"""


@dataclass(eq=False, repr=False)
class AccountBlockRequest(betterproto.Message):
    """Request to block an existing account"""

    uuid: str = betterproto.string_field(1)
    """the uuid of the account to block"""


@dataclass(eq=False, repr=False)
class AccountResetRequest(betterproto.Message):
    """Request to reset an existing account"""

    uuid: str = betterproto.string_field(1)
    """the uuid of the account to reset"""


@dataclass(eq=False, repr=False)
class AccountResetResponse(betterproto.Message):
    """
    Response after account has been reset.   [NOTE] ==== The generated
    activation password in plain text will be returned only once. ====
    """

    activation_password: str = betterproto.string_field(1)
    """the activation password in plain text"""


@dataclass(eq=False, repr=False)
class AccountDeleteRequest(betterproto.Message):
    """Request to delete an Account"""

    uuid: str = betterproto.string_field(1)
    """
    Optional uuid of the account to delete. If left empty the uuid from the
    presented JWT token will be used.
    """


@dataclass(eq=False, repr=False)
class AccountGetRequest(betterproto.Message):
    """Request to query account details"""

    uuid: str = betterproto.string_field(1)
    """
    Optional uuid of the account to get. If left empty the uuid from the
    presented JWT token will be used.
    """


@dataclass(eq=False, repr=False)
class AccountGetResponse(betterproto.Message):
    """Response for the currently authenticated account."""

    account: "_config__.Account" = betterproto.message_field(1)
    """The account details"""

    uuid: str = betterproto.string_field(2)
    """The uuid of the account"""


@dataclass(eq=False, repr=False)
class AccountStoreRequest(betterproto.Message):
    """Store updated account details."""

    uuid: str = betterproto.string_field(1)
    """
    Optional uuid of the account to store. If not provided the uuid from the
    presented JWT token will be used.
    """

    account: "_config__.Account" = betterproto.message_field(2)
    """Optional account data to store."""

    password_update: "AccountStoreRequestPasswordUpdate" = betterproto.message_field(3)
    """Optional password update for the account to store."""


@dataclass(eq=False, repr=False)
class AccountStoreRequestPasswordUpdate(betterproto.Message):
    """Password update message"""

    current: str = betterproto.string_field(1)
    """the current password"""

    new: str = betterproto.string_field(2)
    """the new password"""


@dataclass(eq=False, repr=False)
class AuthenticationLoginRequest(betterproto.Message):
    """Request for authentication"""

    user: "AuthenticationLoginRequestUser" = betterproto.message_field(
        1, group="credentials"
    )
    """Authenticate with user credentials"""

    application: "AuthenticationLoginRequestApplication" = betterproto.message_field(
        2, group="credentials"
    )
    """Authenticate with application key credentials"""


@dataclass(eq=False, repr=False)
class AuthenticationLoginRequestUser(betterproto.Message):
    """User credentials"""

    name: str = betterproto.string_field(1)
    """the username (can be left out in case of default factory state)"""

    password: str = betterproto.string_field(2)
    """the password of the user in plain text"""


@dataclass(eq=False, repr=False)
class AuthenticationLoginRequestApplication(betterproto.Message):
    """Application Key credentials"""

    key: str = betterproto.string_field(1)
    """the key in plain text"""


@dataclass(eq=False, repr=False)
class AuthenticationLoginResponse(betterproto.Message):
    """Response containing the issued JWT token."""

    token: str = betterproto.string_field(1)
    """
    The JWT token which needs to be sent with the key "token" in the metadata
    of a gRPC call. The token is bound to a
    xref:blickfeld/secure/services/session.adoc[].
    """


@dataclass(eq=False, repr=False)
class FirewallSetRequest(betterproto.Message):
    """Set configuration request"""

    config: "_config__.Firewall" = betterproto.message_field(1)
    """Configuration"""


@dataclass(eq=False, repr=False)
class FirewallGetResponse(betterproto.Message):
    """Get configuration response"""

    config: "_config__.Firewall" = betterproto.message_field(1)
    """Configuration"""


@dataclass(eq=False, repr=False)
class DeviceCredentialsSetFactoryRequest(betterproto.Message):
    """
    Request with device credentials   The device credentials should only be set
    once in production.
    """

    device_credentials: "_config__.DeviceCredentials" = betterproto.message_field(1)
    """
    Cryptographic device credentials which were generated by the blickfeld-
    secure-authority.
    """


@dataclass(eq=False, repr=False)
class DeviceCredentialsSetRequest(betterproto.Message):
    """Request with device credentials"""

    device_credentials: "_config__.DeviceCredentials" = betterproto.message_field(1)
    """
    Cryptographic device credentials used to establish secured TLS connection
    between client and Qb2 device.
    """


@dataclass(eq=False, repr=False)
class DeviceCredentialsGetResponse(betterproto.Message):
    """Response with device credentials"""

    device_credentials: "_config__.DeviceCredentials" = betterproto.message_field(1)
    """
    Cryptographic device credentials used to establish secured TLS connection
    between client and Qb2 device.
    """


@dataclass(eq=False, repr=False)
class DeviceCredentialsGetCertificatesResponse(betterproto.Message):
    """Response message which delivers public device certificate"""

    certificates: List["_config__.Certificate"] = betterproto.message_field(1)
    """
    Certificate chain which is deployed at Qb2 device and is used during the
    TLS handshake.
    """


@dataclass(eq=False, repr=False)
class HealthGetResponse(betterproto.Message):
    """Response to Health Get request"""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


@dataclass(eq=False, repr=False)
class HealthWatchResponse(betterproto.Message):
    """Response to Health Watch request"""

    health: "_data__.Health" = betterproto.message_field(1)
    """Health state"""


@dataclass(eq=False, repr=False)
class SessionGetNonceResponse(betterproto.Message):
    """Response for GetNonce request."""

    nonce: str = betterproto.string_field(1)
    """Cryptographic session nonce."""


@dataclass(eq=False, repr=False)
class LicenseSetRequest(betterproto.Message):
    """Request to assign license to this Qb2"""

    token: str = betterproto.string_field(1)
    """JWT license string"""


@dataclass(eq=False, repr=False)
class LicenseGetResponse(betterproto.Message):
    """Response containing licensed product variant and metadata"""

    license: "_data__.License" = betterproto.message_field(1)
    """The configured license"""


class ApplicationKey(betterproto.ServiceStub):
    """Service for managing application keys."""

    async def async_list(
        self,
        *,
        account_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ApplicationKeyListResponse":
        """List existing application keys for the authenticated account."""

        request = ApplicationKeyListRequest()
        request.account_uuid = account_uuid

        return await self._unary_unary(
            "/blickfeld.secure.services.ApplicationKey/List",
            request,
            ApplicationKeyListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        *,
        account_uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ApplicationKeyListResponse":
        """List existing application keys for the authenticated account."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                account_uuid=account_uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_create(
        self,
        *,
        account_uuid: str = "",
        application_key: "_config__.ApplicationKey" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ApplicationKeyCreateResponse":
        """Create an application key for the authenticated account."""

        request = ApplicationKeyCreateRequest()
        request.account_uuid = account_uuid
        if application_key is not None:
            request.application_key = application_key

        return await self._unary_unary(
            "/blickfeld.secure.services.ApplicationKey/Create",
            request,
            ApplicationKeyCreateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def create(
        self,
        *,
        account_uuid: str = "",
        application_key: "_config__.ApplicationKey" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "ApplicationKeyCreateResponse":
        """Create an application key for the authenticated account."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_create(
                account_uuid=account_uuid,
                application_key=application_key,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete an application key from the authenticated account."""

        request = ApplicationKeyDeleteRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.secure.services.ApplicationKey/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete an application key from the authenticated account."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Account(betterproto.ServiceStub):
    """
    This service supports the complete lifecycle of accounts which are managed
    by the current Qb2 device.
    """

    async def async_list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "AccountListResponse":
        """List registered accounts."""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/List",
            request,
            AccountListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "AccountListResponse":
        """List registered accounts."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_create(
        self,
        *,
        account: "_config__.Account" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AccountCreateResponse":
        """Create a new account."""

        request = AccountCreateRequest()
        if account is not None:
            request.account = account

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/Create",
            request,
            AccountCreateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def create(
        self,
        *,
        account: "_config__.Account" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AccountCreateResponse":
        """Create a new account."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_create(
                account=account,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_block(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Block account by setting the state to BLOCKED. Use the Reset method to
        un-block the account.
        """

        request = AccountBlockRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/Block",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def block(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Block account by setting the state to BLOCKED. Use the Reset method to
        un-block the account.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_block(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_reset(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AccountResetResponse":
        """
        Reset account by setting the state to WAITING_FOR_ACTIVATION &
        generating a new activation password.
        """

        request = AccountResetRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/Reset",
            request,
            AccountResetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def reset(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AccountResetResponse":
        """
        Reset account by setting the state to WAITING_FOR_ACTIVATION &
        generating a new activation password.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_reset(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete an Account."""

        request = AccountDeleteRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Delete an Account."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AccountGetResponse":
        """Get Account information."""

        request = AccountGetRequest()
        request.uuid = uuid

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/Get",
            request,
            AccountGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        *,
        uuid: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AccountGetResponse":
        """Get Account information."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                uuid=uuid,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_store(
        self,
        *,
        uuid: str = "",
        account: "_config__.Account" = None,
        password_update: "AccountStoreRequestPasswordUpdate" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Store updated account data.   [NOTE] ==== During account activation
        this method can be accessed using a JWT token with access level
        `LEVEL_PUBLIC`. It will fail with `UNAUTHENTICATED` if no JWT token is
        presented. ====
        """

        request = AccountStoreRequest()
        request.uuid = uuid
        if account is not None:
            request.account = account
        if password_update is not None:
            request.password_update = password_update

        return await self._unary_unary(
            "/blickfeld.secure.services.Account/Store",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store(
        self,
        *,
        uuid: str = "",
        account: "_config__.Account" = None,
        password_update: "AccountStoreRequestPasswordUpdate" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Store updated account data.   [NOTE] ==== During account activation
        this method can be accessed using a JWT token with access level
        `LEVEL_PUBLIC`. It will fail with `UNAUTHENTICATED` if no JWT token is
        presented. ====
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store(
                uuid=uuid,
                account=account,
                password_update=password_update,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Authentication(betterproto.ServiceStub):
    """Service for authentication to the Qb2 device."""

    async def async_login(
        self,
        *,
        user: "AuthenticationLoginRequestUser" = None,
        application: "AuthenticationLoginRequestApplication" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AuthenticationLoginResponse":
        """
        User-Accounts managed by Qb2 have to use the Login method with an
        existing account in order to receive a JWT Token for access to
        priviliged parts of the device API.   The factory user has the username
        "admin". The password is printed on the device label.
        """

        request = AuthenticationLoginRequest()
        if user is not None:
            request.user = user
        if application is not None:
            request.application = application

        return await self._unary_unary(
            "/blickfeld.secure.services.Authentication/Login",
            request,
            AuthenticationLoginResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def login(
        self,
        *,
        user: "AuthenticationLoginRequestUser" = None,
        application: "AuthenticationLoginRequestApplication" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "AuthenticationLoginResponse":
        """
        User-Accounts managed by Qb2 have to use the Login method with an
        existing account in order to receive a JWT Token for access to
        priviliged parts of the device API.   The factory user has the username
        "admin". The password is printed on the device label.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_login(
                user=user,
                application=application,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Firewall(betterproto.ServiceStub):
    """Service for configuring firewall parameters"""

    async def async_set(
        self,
        *,
        config: "_config__.Firewall" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set configuration"""

        request = FirewallSetRequest()
        if config is not None:
            request.config = config

        return await self._unary_unary(
            "/blickfeld.secure.services.Firewall/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        config: "_config__.Firewall" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                config=config,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FirewallGetResponse":
        """Get configuration"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.Firewall/Get",
            request,
            FirewallGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FirewallGetResponse":
        """Get configuration"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class DeviceCredentials(betterproto.ServiceStub):
    """
    Set device credentials for the secure TLS connection of the Qb2 device.
    """

    async def async_set(
        self,
        *,
        device_credentials: "_config__.DeviceCredentials" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set device credentials.   [WARN] This will override the factory SSL
        certificate which is provided by the device. It will cause problems
        with inter-device communication, authentication, and Blickfeld client
        libraries. Blickfeld client libraries expect a root certificate which
        is signed by the Blickfeld device root authority. The expected root
        certificate must then be manually provided via configuration options in
        the client libraries.
        """

        request = DeviceCredentialsSetRequest()
        if device_credentials is not None:
            request.device_credentials = device_credentials

        return await self._unary_unary(
            "/blickfeld.secure.services.DeviceCredentials/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        device_credentials: "_config__.DeviceCredentials" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set device credentials.   [WARN] This will override the factory SSL
        certificate which is provided by the device. It will cause problems
        with inter-device communication, authentication, and Blickfeld client
        libraries. Blickfeld client libraries expect a root certificate which
        is signed by the Blickfeld device root authority. The expected root
        certificate must then be manually provided via configuration options in
        the client libraries.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                device_credentials=device_credentials,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get_certificates(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "DeviceCredentialsGetCertificatesResponse":
        """Get device credentials certificates."""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.DeviceCredentials/GetCertificates",
            request,
            DeviceCredentialsGetCertificatesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_certificates(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "DeviceCredentialsGetCertificatesResponse":
        """Get device credentials certificates."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_certificates(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_clear(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Clear device credentials. The device credentials will reset to the
        factory device credentials.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.DeviceCredentials/Clear",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def clear(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Clear device credentials. The device credentials will reset to the
        factory device credentials.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_clear(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Health(betterproto.ServiceStub):
    """
    The health service provides methods to monitor operational status of the
    secure module
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the secure module"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.Health/Get",
            request,
            HealthGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "HealthGetResponse":
        """Returns the current health status of the secure module"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> AsyncIterator["HealthWatchResponse"]:
        """
        Continously returns the current health status of the secure module
        """

        request = betterproto_lib_google_protobuf.Empty()

        async for response in self._unary_stream(
            "/blickfeld.secure.services.Health/Watch",
            request,
            HealthWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> Iterator["HealthWatchResponse"]:
        """
        Continously returns the current health status of the secure module
        """

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj


class Session(betterproto.ServiceStub):
    """
    Session service which implements all methods for session-bound security
    features.   A session is recreated after the following events: * Power-
    cycle of the device * Every day (UTC) * Manual invalidation via gRPC call
    """

    async def async_get_nonce(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "SessionGetNonceResponse":
        """Get cryptographic session nonce."""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.Session/GetNonce",
            request,
            SessionGetNonceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_nonce(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "SessionGetNonceResponse":
        """Get cryptographic session nonce."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_nonce(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_invalidate(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Invalidates the current session. A new session will be created
        automatically.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.Session/Invalidate",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def invalidate(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Invalidates the current session. A new session will be created
        automatically.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_invalidate(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class License(betterproto.ServiceStub):
    """
    License service to assign a product variant and metadata to a Qb2.   The
    license is cryptographically singed and configured during production. A
    configured license can only be replaced by another valid license for this
    device.   A license contains the specific product variant and can
    additionally contain arbitrary metadata for the specfic customer or
    application.
    """

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "LicenseGetResponse":
        """Get configured license"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.secure.services.License/Get",
            request,
            LicenseGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "LicenseGetResponse":
        """Get configured license"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_set(
        self,
        *,
        token: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set license"""

        request = LicenseSetRequest()
        request.token = token

        return await self._unary_unary(
            "/blickfeld.secure.services.License/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        token: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Set license"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                token=token,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/compute_module/data/health.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    """Describes the health of the compute module"""

    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is STATE_OK."""

    error_flags: "HealthErrorFlags" = betterproto.message_field(3)
    """Error flags"""

    cpu_performance: "HealthCpuPerformance" = betterproto.message_field(4)
    """CPU performance"""

    memory_utilization: float = betterproto.float_field(5)
    """
    Memory utilization of the compute module.   The value range is between 0
    and 1.
    """

    uptime: "HealthUptime" = betterproto.message_field(6)
    """Uptime"""


@dataclass(eq=False, repr=False)
class HealthErrorFlags(betterproto.Message):
    """Error flags"""

    high_cpu_load: bool = betterproto.bool_field(1)
    """
    The average CPU load of sensor is too high. The processing and frame rate
    will be automatically throttled.
    """

    high_temperature: bool = betterproto.bool_field(2)
    """
    The temperature of the device is too high.   The CPU frequency will be
    throttled automatically. The processing capability might be limited. Note
    that an operation outside of the specified temperature range, can cause
    permanent damage to the sensor.
    """


@dataclass(eq=False, repr=False)
class HealthCpuPerformance(betterproto.Message):
    """Performance statistics of the CPU"""

    total_load: float = betterproto.float_field(1)
    """Average CPU load of all cores.   The value range is between 0 and 1."""

    individual_loads: List[float] = betterproto.float_field(2)
    """Individual load per core.   The value range is between 0 and 1."""

    frequency: float = betterproto.float_field(3)
    """Current frequency"""

    temperature: float = betterproto.float_field(4)
    """Current temperature"""


@dataclass(eq=False, repr=False)
class HealthUptime(betterproto.Message):
    """Uptime of the device"""

    since_boot: int = betterproto.uint64_field(1)
    """Uptime since the last boot"""

    lifetime: int = betterproto.uint64_field(2)
    """
    Uptime over the whole lifetime.   NOTE: The counting started with the
    firmware v2.3.
    """



import asyncio
from abc import ABC
from typing import (
    TYPE_CHECKING,
    AsyncIterable,
    AsyncIterator,
    Collection,
    Iterable,
    Mapping,
    Optional,
    Tuple,
    Type,
    Union,
)

import grpclib.const


if TYPE_CHECKING:
    from grpclib.client import Channel
    from grpclib.metadata import Deadline

    from .._types import (
        ST,
        IProtoMessage,
        Message,
        T,
    )


Value = Union[str, bytes]
MetadataLike = Union[Mapping[str, Value], Collection[Tuple[str, Value]]]
MessageSource = Union[Iterable["IProtoMessage"], AsyncIterable["IProtoMessage"]]


class ServiceStub(ABC):
    """
    Base class for async gRPC clients.
    """

    def __init__(
        self,
        channel: "Channel",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional[MetadataLike] = None,
    ) -> None:
        self.channel = channel
        self.timeout = timeout
        self.deadline = deadline
        self.metadata = metadata

    def __resolve_request_kwargs(
        self,
        timeout: Optional[float],
        deadline: Optional["Deadline"],
        metadata: Optional[MetadataLike],
    ):
        return {
            "timeout": self.timeout if timeout is None else timeout,
            "deadline": self.deadline if deadline is None else deadline,
            "metadata": self.metadata if metadata is None else metadata,
        }

    async def _unary_unary(
        self,
        route: str,
        request: "IProtoMessage",
        response_type: Type["T"],
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional[MetadataLike] = None,
    ) -> "T":
        """Make a unary request and return the response."""
        async with self.channel.request(
            route,
            grpclib.const.Cardinality.UNARY_UNARY,
            type(request),
            response_type,
            **self.__resolve_request_kwargs(timeout, deadline, metadata),
        ) as stream:
            await stream.send_message(request, end=True)
            response = await stream.recv_message()
        assert response is not None
        return response

    async def _unary_stream(
        self,
        route: str,
        request: "IProtoMessage",
        response_type: Type["T"],
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional[MetadataLike] = None,
    ) -> AsyncIterator["T"]:
        """Make a unary request and return the stream response iterator."""
        async with self.channel.request(
            route,
            grpclib.const.Cardinality.UNARY_STREAM,
            type(request),
            response_type,
            **self.__resolve_request_kwargs(timeout, deadline, metadata),
        ) as stream:
            await stream.send_message(request, end=True)
            async for message in stream:
                yield message

    async def _stream_unary(
        self,
        route: str,
        request_iterator: MessageSource,
        request_type: Type["IProtoMessage"],
        response_type: Type["T"],
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional[MetadataLike] = None,
    ) -> "T":
        """Make a stream request and return the response."""
        async with self.channel.request(
            route,
            grpclib.const.Cardinality.STREAM_UNARY,
            request_type,
            response_type,
            **self.__resolve_request_kwargs(timeout, deadline, metadata),
        ) as stream:
            await self._send_messages(stream, request_iterator)
            response = await stream.recv_message()
        assert response is not None
        return response

    async def _stream_stream(
        self,
        route: str,
        request_iterator: MessageSource,
        request_type: Type["IProtoMessage"],
        response_type: Type["T"],
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional[MetadataLike] = None,
    ) -> AsyncIterator["T"]:
        """
        Make a stream request and return an AsyncIterator to iterate over response
        messages.
        """
        async with self.channel.request(
            route,
            grpclib.const.Cardinality.STREAM_STREAM,
            request_type,
            response_type,
            **self.__resolve_request_kwargs(timeout, deadline, metadata),
        ) as stream:
            await stream.send_request()
            sending_task = asyncio.ensure_future(
                self._send_messages(stream, request_iterator)
            )
            try:
                async for response in stream:
                    yield response
            except:
                sending_task.cancel()
                raise

    @staticmethod
    async def _send_messages(stream, messages: MessageSource):
        if isinstance(messages, AsyncIterable):
            async for message in messages:
                await stream.send_message(message)
        else:
            for message in messages:
                await stream.send_message(message)
        await stream.end()





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/secure/config/account.proto, blickfeld/secure/config/application_key.proto, blickfeld/secure/config/certificate.proto, blickfeld/secure/config/device_credentials.proto, blickfeld/secure/config/firewall.proto, blickfeld/secure/config/private_key.proto, blickfeld/secure/config/public_key.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ... import base as __base__


class AccountState(betterproto.Enum):
    """Account state"""

    STATE_UNSPECIFIED = 0
    """status is unspecified."""

    STATE_WAITING_FOR_ACTIVATION = 1
    """waiting for activation, account has to be activated by the user."""

    STATE_ACTIVE = 2
    """active, account can be used to authenticate."""

    STATE_BLOCKED = 3
    """blocked, account can not be used for authentication."""


@dataclass(eq=False, repr=False)
class ApplicationKey(betterproto.Message):
    """An application key is linked to a user account."""

    purpose: str = betterproto.string_field(1)
    """
    Human readable purpose for this application key. It is recommended to
    briefly describe the purpose of the application key here. The field should
    help users to quickly identify different application keys but can also be
    left empty.
    """

    access_level: "__base__.AccessControlLevel" = betterproto.enum_field(2)
    """Access level which is granted to this application key."""

    access_read_only: bool = betterproto.bool_field(7)
    """If true, this application key has read-only access permissions."""

    access_http: bool = betterproto.bool_field(8)
    """
    If true, allow http basic access authentication using this key. This option
    is required for ONVIF and RTSP video streaming.
    """


@dataclass(eq=False, repr=False)
class Account(betterproto.Message):
    """
    An account identifies a single user defined on the Qb2 device. By
    presenting credentials linked to an account a client can be authenticated.
    """

    name: str = betterproto.string_field(1)
    """Account name. This field is used as the username during login."""

    access_level: "__base__.AccessControlLevel" = betterproto.enum_field(2)
    """Access level which is granted to this account."""

    access_read_only: bool = betterproto.bool_field(8)
    """If true, this account has read-only access permissions."""

    failed_login_attempts: int = betterproto.uint32_field(5)
    """
    Counter for failed login attempts. If this value exceeds a configured
    threshold the account will be blocked.
    """

    state: "AccountState" = betterproto.enum_field(6)
    """The account state."""


@dataclass(eq=False, repr=False)
class Certificate(betterproto.Message):
    """Public key certificate during the TLS handshake process"""

    binary: bytes = betterproto.bytes_field(1)
    """Certificate bytes (DER format)"""

    type: str = betterproto.string_field(2)
    """
    String giving information about which key type was used for generation
    (e.g. RSA, key length)
    """

    serial: str = betterproto.string_field(3)
    """
    Serial number of certificate in padded hex format. Automatically assigned
    by the service when issued.
    """


@dataclass(eq=False, repr=False)
class PrivateKey(betterproto.Message):
    """
    Private device key which is deployed on the Qb2 device and used for TLS
    authentification by Daedalus Proxy
    """

    binary: bytes = betterproto.bytes_field(1)
    """Key bytes as binary"""

    type: str = betterproto.string_field(2)
    """String giving information about which key type is used"""


@dataclass(eq=False, repr=False)
class DeviceCredentials(betterproto.Message):
    """
    Cryptographic device credentials which are used to establish secure TLS
    connection between client and Qb2 and get access to its APIs
    """

    certificates: List["Certificate"] = betterproto.message_field(1)
    """
    Certificate chain which is deployed at Qb2 device and is used during the
    TLS handshake. Chain includes device certificate and corresponding issuer
    batch certificate.
    """

    key: "PrivateKey" = betterproto.message_field(2)
    """
    Device private key which is deployed at Qb2 device. Should never leave Qb2
    device after being deployed.
    """


@dataclass(eq=False, repr=False)
class Firewall(betterproto.Message):
    """Configuration options for the firewall"""

    disable_insecure_network_ports: bool = betterproto.bool_field(1)
    """
    Disable all insecure, unencrypted & unauthenticated, network interface
    ports.   [NOTE] This might break compatibility with client applications
    e.g. a normal  HTTP call to http://qb2-xxxxx will fail.
    """

    flow_port_forwardings: List["FirewallPortForwarding"] = betterproto.message_field(2)
    """
    The given ports are forwarded to custom applications in the blickfeld flow.
    """


@dataclass(eq=False, repr=False)
class FirewallPortForwarding(betterproto.Message):
    """Rule for port forwarding"""

    port: int = betterproto.uint32_field(1)
    """
    Port which should be forwarded.   NOTE: Due to security policies, only
    ports between 10000 to 10099 are allowed.
    """

    from_ip_network: str = betterproto.string_field(2)
    """
    The port forwarding is only allowed for the given IP address or subnet.
    The address should be specified in dot-decimal format (quad-dotted notation
    of four decimal integers, ranging from 0 to 255 each) including netmask
    prefix as follows "ip_address/prefix".
    """


@dataclass(eq=False, repr=False)
class PublicKey(betterproto.Message):
    """
    Public key which is used for verification or encryption. Verification keys
    are either hard-coded in applications or fused in hardware registers.
    """

    binary: bytes = betterproto.bytes_field(1)
    """Key bytes as binary"""

    type: str = betterproto.string_field(2)
    """String giving information about which key type is used"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: google/protobuf/any.proto, google/protobuf/api.proto, google/protobuf/descriptor.proto, google/protobuf/duration.proto, google/protobuf/empty.proto, google/protobuf/field_mask.proto, google/protobuf/source_context.proto, google/protobuf/struct.proto, google/protobuf/timestamp.proto, google/protobuf/type.proto, google/protobuf/wrappers.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from typing import (
    Dict,
    List,
)

import blickfeld_qb2.betterproto as betterproto
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase


class Syntax(betterproto.Enum):
    """The syntax in which a protocol buffer element is defined."""

    # Syntax `proto2`.
    SYNTAX_PROTO2 = 0
    # Syntax `proto3`.
    SYNTAX_PROTO3 = 1


class FieldKind(betterproto.Enum):
    TYPE_UNKNOWN = 0
    TYPE_DOUBLE = 1
    TYPE_FLOAT = 2
    TYPE_INT64 = 3
    TYPE_UINT64 = 4
    TYPE_INT32 = 5
    TYPE_FIXED64 = 6
    TYPE_FIXED32 = 7
    TYPE_BOOL = 8
    TYPE_STRING = 9
    TYPE_GROUP = 10
    TYPE_MESSAGE = 11
    TYPE_BYTES = 12
    TYPE_UINT32 = 13
    TYPE_ENUM = 14
    TYPE_SFIXED32 = 15
    TYPE_SFIXED64 = 16
    TYPE_SINT32 = 17
    TYPE_SINT64 = 18


class FieldCardinality(betterproto.Enum):
    CARDINALITY_UNKNOWN = 0
    CARDINALITY_OPTIONAL = 1
    CARDINALITY_REQUIRED = 2
    CARDINALITY_REPEATED = 3


class FieldDescriptorProtoType(betterproto.Enum):
    TYPE_DOUBLE = 1
    TYPE_FLOAT = 2
    TYPE_INT64 = 3
    TYPE_UINT64 = 4
    TYPE_INT32 = 5
    TYPE_FIXED64 = 6
    TYPE_FIXED32 = 7
    TYPE_BOOL = 8
    TYPE_STRING = 9
    TYPE_GROUP = 10
    TYPE_MESSAGE = 11
    TYPE_BYTES = 12
    TYPE_UINT32 = 13
    TYPE_ENUM = 14
    TYPE_SFIXED32 = 15
    TYPE_SFIXED64 = 16
    TYPE_SINT32 = 17
    TYPE_SINT64 = 18


class FieldDescriptorProtoLabel(betterproto.Enum):
    LABEL_OPTIONAL = 1
    LABEL_REQUIRED = 2
    LABEL_REPEATED = 3


class FileOptionsOptimizeMode(betterproto.Enum):
    SPEED = 1
    CODE_SIZE = 2
    LITE_RUNTIME = 3


class FieldOptionsCType(betterproto.Enum):
    STRING = 0
    CORD = 1
    STRING_PIECE = 2


class FieldOptionsJsType(betterproto.Enum):
    JS_NORMAL = 0
    JS_STRING = 1
    JS_NUMBER = 2


class MethodOptionsIdempotencyLevel(betterproto.Enum):
    IDEMPOTENCY_UNKNOWN = 0
    NO_SIDE_EFFECTS = 1
    IDEMPOTENT = 2


class NullValue(betterproto.Enum):
    """
    `NullValue` is a singleton enumeration to represent the null value for the
    `Value` type union.  The JSON representation for `NullValue` is JSON
    `null`.
    """

    # Null value.
    NULL_VALUE = 0


@dataclass(eq=False, repr=False)
class Any(betterproto.Message):
    """
    `Any` contains an arbitrary serialized protocol buffer message along with a
    URL that describes the type of the serialized message. Protobuf library
    provides support to pack/unpack Any values in the form of utility functions
    or additional generated methods of the Any type. Example 1: Pack and unpack
    a message in C++.     Foo foo = ...;     Any any;     any.PackFrom(foo);
    ...     if (any.UnpackTo(&foo)) {       ...     } Example 2: Pack and
    unpack a message in Java.     Foo foo = ...;     Any any = Any.pack(foo);
    ...     if (any.is(Foo.class)) {       foo = any.unpack(Foo.class);     }
    Example 3: Pack and unpack a message in Python.     foo = Foo(...)     any
    = Any()     any.Pack(foo)     ...     if any.Is(Foo.DESCRIPTOR):
    any.Unpack(foo)       ...  Example 4: Pack and unpack a message in Go
    foo := &pb.Foo{...}      any, err := anypb.New(foo)      if err != nil {
    ...      }      ...      foo := &pb.Foo{}      if err :=
    any.UnmarshalTo(foo); err != nil {        ...      } The pack methods
    provided by protobuf library will by default use
    'type.googleapis.com/full.type.name' as the type URL and the unpack methods
    only use the fully qualified type name after the last '/' in the type URL,
    for example "foo.bar.com/x/y.z" will yield type name "y.z". JSON ==== The
    JSON representation of an `Any` value uses the regular representation of
    the deserialized, embedded message, with an additional field `@type` which
    contains the type URL. Example:     package google.profile;     message
    Person {       string first_name = 1;       string last_name = 2;     }
    {       "@type": "type.googleapis.com/google.profile.Person",
    "firstName": <string>,       "lastName": <string>     } If the embedded
    message type is well-known and has a custom JSON representation, that
    representation will be embedded adding a field `value` which holds the
    custom JSON in addition to the `@type` field. Example (for message
    [google.protobuf.Duration][]):     {       "@type":
    "type.googleapis.com/google.protobuf.Duration",       "value": "1.212s"
    }
    """

    # A URL/resource name that uniquely identifies the type of the serialized
    # protocol buffer message. This string must contain at least one "/"
    # character. The last segment of the URL's path must represent the fully
    # qualified name of the type (as in `path/google.protobuf.Duration`). The
    # name should be in a canonical form (e.g., leading "." is not accepted). In
    # practice, teams usually precompile into the binary all types that they
    # expect it to use in the context of Any. However, for URLs which use the
    # scheme `http`, `https`, or no scheme, one can optionally set up a type
    # server that maps type URLs to message definitions as follows: * If no
    # scheme is provided, `https` is assumed. * An HTTP GET on the URL must yield
    # a [google.protobuf.Type][]   value in binary format, or produce an error. *
    # Applications are allowed to cache lookup results based on the   URL, or
    # have them precompiled into a binary to avoid any   lookup. Therefore,
    # binary compatibility needs to be preserved   on changes to types. (Use
    # versioned type names to manage   breaking changes.) Note: this
    # functionality is not currently available in the official protobuf release,
    # and it is not used for type URLs beginning with type.googleapis.com.
    # Schemes other than `http`, `https` (or the empty scheme) might be used with
    # implementation specific semantics.
    type_url: str = betterproto.string_field(1)
    # Must be a valid serialized protocol buffer of the above specified type.
    value: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class SourceContext(betterproto.Message):
    """
    `SourceContext` represents information about the source of a protobuf
    element, like the file in which it is defined.
    """

    # The path-qualified name of the .proto file that contained the associated
    # protobuf element.  For example: `"google/protobuf/source_context.proto"`.
    file_name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class Type(betterproto.Message):
    """A protocol buffer message type."""

    # The fully qualified message name.
    name: str = betterproto.string_field(1)
    # The list of fields.
    fields: List["Field"] = betterproto.message_field(2)
    # The list of types appearing in `oneof` definitions in this type.
    oneofs: List[str] = betterproto.string_field(3)
    # The protocol buffer options.
    options: List["Option"] = betterproto.message_field(4)
    # The source context.
    source_context: "SourceContext" = betterproto.message_field(5)
    # The source syntax.
    syntax: "Syntax" = betterproto.enum_field(6)


@dataclass(eq=False, repr=False)
class Field(betterproto.Message):
    """A single field of a message type."""

    # The field type.
    kind: "FieldKind" = betterproto.enum_field(1)
    # The field cardinality.
    cardinality: "FieldCardinality" = betterproto.enum_field(2)
    # The field number.
    number: int = betterproto.int32_field(3)
    # The field name.
    name: str = betterproto.string_field(4)
    # The field type URL, without the scheme, for message or enumeration types.
    # Example: `"type.googleapis.com/google.protobuf.Timestamp"`.
    type_url: str = betterproto.string_field(6)
    # The index of the field type in `Type.oneofs`, for message or enumeration
    # types. The first type has index 1; zero means the type is not in the list.
    oneof_index: int = betterproto.int32_field(7)
    # Whether to use alternative packed wire representation.
    packed: bool = betterproto.bool_field(8)
    # The protocol buffer options.
    options: List["Option"] = betterproto.message_field(9)
    # The field JSON name.
    json_name: str = betterproto.string_field(10)
    # The string value of the default value of this field. Proto2 syntax only.
    default_value: str = betterproto.string_field(11)


@dataclass(eq=False, repr=False)
class Enum(betterproto.Message):
    """Enum type definition."""

    # Enum type name.
    name: str = betterproto.string_field(1)
    # Enum value definitions.
    enumvalue: List["EnumValue"] = betterproto.message_field(
        2, wraps=betterproto.TYPE_ENUM
    )
    # Protocol buffer options.
    options: List["Option"] = betterproto.message_field(3)
    # The source context.
    source_context: "SourceContext" = betterproto.message_field(4)
    # The source syntax.
    syntax: "Syntax" = betterproto.enum_field(5)


@dataclass(eq=False, repr=False)
class EnumValue(betterproto.Message):
    """Enum value definition."""

    # Enum value name.
    name: str = betterproto.string_field(1)
    # Enum value number.
    number: int = betterproto.int32_field(2)
    # Protocol buffer options.
    options: List["Option"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Option(betterproto.Message):
    """
    A protocol buffer option, which can be attached to a message, field,
    enumeration, etc.
    """

    # The option's name. For protobuf built-in options (options defined in
    # descriptor.proto), this is the short name. For example, `"map_entry"`. For
    # custom options, it should be the fully-qualified name. For example,
    # `"google.api.http"`.
    name: str = betterproto.string_field(1)
    # The option's value packed in an Any message. If the value is a primitive,
    # the corresponding wrapper type defined in google/protobuf/wrappers.proto
    # should be used. If the value is an enum, it should be stored as an int32
    # value using the google.protobuf.Int32Value type.
    value: "Any" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Api(betterproto.Message):
    """
    Api is a light-weight descriptor for an API Interface. Interfaces are also
    described as "protocol buffer services" in some contexts, such as by the
    "service" keyword in a .proto file, but they are different from API
    Services, which represent a concrete implementation of an interface as
    opposed to simply a description of methods and bindings. They are also
    sometimes simply referred to as "APIs" in other contexts, such as the name
    of this message itself. See https://cloud.google.com/apis/design/glossary
    for detailed terminology.
    """

    # The fully qualified name of this interface, including package name followed
    # by the interface's simple name.
    name: str = betterproto.string_field(1)
    # The methods of this interface, in unspecified order.
    methods: List["Method"] = betterproto.message_field(2)
    # Any metadata attached to the interface.
    options: List["Option"] = betterproto.message_field(3)
    # A version string for this interface. If specified, must have the form
    # `major-version.minor-version`, as in `1.10`. If the minor version is
    # omitted, it defaults to zero. If the entire version field is empty, the
    # major version is derived from the package name, as outlined below. If the
    # field is not empty, the version in the package name will be verified to be
    # consistent with what is provided here. The versioning schema uses [semantic
    # versioning](http://semver.org) where the major version number indicates a
    # breaking change and the minor version an additive, non-breaking change.
    # Both version numbers are signals to users what to expect from different
    # versions, and should be carefully chosen based on the product plan. The
    # major version is also reflected in the package name of the interface, which
    # must end in `v<major-version>`, as in `google.feature.v1`. For major
    # versions 0 and 1, the suffix can be omitted. Zero major versions must only
    # be used for experimental, non-GA interfaces.
    version: str = betterproto.string_field(4)
    # Source context for the protocol buffer service represented by this message.
    source_context: "SourceContext" = betterproto.message_field(5)
    # Included interfaces. See [Mixin][].
    mixins: List["Mixin"] = betterproto.message_field(6)
    # The source syntax of the service.
    syntax: "Syntax" = betterproto.enum_field(7)


@dataclass(eq=False, repr=False)
class Method(betterproto.Message):
    """Method represents a method of an API interface."""

    # The simple name of this method.
    name: str = betterproto.string_field(1)
    # A URL of the input message type.
    request_type_url: str = betterproto.string_field(2)
    # If true, the request is streamed.
    request_streaming: bool = betterproto.bool_field(3)
    # The URL of the output message type.
    response_type_url: str = betterproto.string_field(4)
    # If true, the response is streamed.
    response_streaming: bool = betterproto.bool_field(5)
    # Any metadata attached to the method.
    options: List["Option"] = betterproto.message_field(6)
    # The source syntax of this method.
    syntax: "Syntax" = betterproto.enum_field(7)


@dataclass(eq=False, repr=False)
class Mixin(betterproto.Message):
    """
    Declares an API Interface to be included in this interface. The including
    interface must redeclare all the methods from the included interface, but
    documentation and options are inherited as follows: - If after comment and
    whitespace stripping, the documentation   string of the redeclared method
    is empty, it will be inherited   from the original method. - Each
    annotation belonging to the service config (http,   visibility) which is
    not set in the redeclared method will be   inherited. - If an http
    annotation is inherited, the path pattern will be   modified as follows.
    Any version prefix will be replaced by the   version of the including
    interface plus the [root][] path if   specified. Example of a simple mixin:
    package google.acl.v1;     service AccessControl {       // Get the
    underlying ACL object.       rpc GetAcl(GetAclRequest) returns (Acl) {
    option (google.api.http).get = "/v1/{resource=**}:getAcl";       }     }
    package google.storage.v2;     service Storage {       rpc
    GetAcl(GetAclRequest) returns (Acl);       // Get a data record.       rpc
    GetData(GetDataRequest) returns (Data) {         option
    (google.api.http).get = "/v2/{resource=**}";       }     } Example of a
    mixin configuration:     apis:     - name: google.storage.v2.Storage
    mixins:       - name: google.acl.v1.AccessControl The mixin construct
    implies that all methods in `AccessControl` are also declared with same
    name and request/response types in `Storage`. A documentation generator or
    annotation processor will see the effective `Storage.GetAcl` method after
    inheriting documentation and annotations as follows:     service Storage {
    // Get the underlying ACL object.       rpc GetAcl(GetAclRequest) returns
    (Acl) {         option (google.api.http).get = "/v2/{resource=**}:getAcl";
    }       ...     } Note how the version in the path pattern changed from
    `v1` to `v2`. If the `root` field in the mixin is specified, it should be a
    relative path under which inherited HTTP paths are placed. Example:
    apis:     - name: google.storage.v2.Storage       mixins:       - name:
    google.acl.v1.AccessControl         root: acls This implies the following
    inherited HTTP annotation:     service Storage {       // Get the
    underlying ACL object.       rpc GetAcl(GetAclRequest) returns (Acl) {
    option (google.api.http).get = "/v2/acls/{resource=**}:getAcl";       }
    ...     }
    """

    # The fully qualified name of the interface which is included.
    name: str = betterproto.string_field(1)
    # If non-empty specifies a path under which inherited HTTP paths are rooted.
    root: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class FileDescriptorSet(betterproto.Message):
    """
    The protocol compiler can output a FileDescriptorSet containing the .proto
    files it parses.
    """

    file: List["FileDescriptorProto"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class FileDescriptorProto(betterproto.Message):
    """Describes a complete .proto file."""

    name: str = betterproto.string_field(1)
    package: str = betterproto.string_field(2)
    # Names of files imported by this file.
    dependency: List[str] = betterproto.string_field(3)
    # Indexes of the public imported files in the dependency list above.
    public_dependency: List[int] = betterproto.int32_field(10)
    # Indexes of the weak imported files in the dependency list. For Google-
    # internal migration only. Do not use.
    weak_dependency: List[int] = betterproto.int32_field(11)
    # All top-level definitions in this file.
    message_type: List["DescriptorProto"] = betterproto.message_field(4)
    enum_type: List["EnumDescriptorProto"] = betterproto.message_field(5)
    service: List["ServiceDescriptorProto"] = betterproto.message_field(6)
    extension: List["FieldDescriptorProto"] = betterproto.message_field(7)
    options: "FileOptions" = betterproto.message_field(8)
    # This field contains optional information about the original source code.
    # You may safely remove this entire field without harming runtime
    # functionality of the descriptors -- the information is needed only by
    # development tools.
    source_code_info: "SourceCodeInfo" = betterproto.message_field(9)
    # The syntax of the proto file. The supported values are "proto2" and
    # "proto3".
    syntax: str = betterproto.string_field(12)


@dataclass(eq=False, repr=False)
class DescriptorProto(betterproto.Message):
    """Describes a message type."""

    name: str = betterproto.string_field(1)
    field: List["FieldDescriptorProto"] = betterproto.message_field(2)
    extension: List["FieldDescriptorProto"] = betterproto.message_field(6)
    nested_type: List["DescriptorProto"] = betterproto.message_field(3)
    enum_type: List["EnumDescriptorProto"] = betterproto.message_field(4)
    extension_range: List["DescriptorProtoExtensionRange"] = betterproto.message_field(
        5
    )
    oneof_decl: List["OneofDescriptorProto"] = betterproto.message_field(8)
    options: "MessageOptions" = betterproto.message_field(7)
    reserved_range: List["DescriptorProtoReservedRange"] = betterproto.message_field(9)
    # Reserved field names, which may not be used by fields in the same message.
    # A given name may only be reserved once.
    reserved_name: List[str] = betterproto.string_field(10)


@dataclass(eq=False, repr=False)
class DescriptorProtoExtensionRange(betterproto.Message):
    start: int = betterproto.int32_field(1)
    end: int = betterproto.int32_field(2)
    options: "ExtensionRangeOptions" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class DescriptorProtoReservedRange(betterproto.Message):
    """
    Range of reserved tag numbers. Reserved tag numbers may not be used by
    fields or extension ranges in the same message. Reserved ranges may not
    overlap.
    """

    start: int = betterproto.int32_field(1)
    end: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class ExtensionRangeOptions(betterproto.Message):
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class FieldDescriptorProto(betterproto.Message):
    """Describes a field within a message."""

    name: str = betterproto.string_field(1)
    number: int = betterproto.int32_field(3)
    label: "FieldDescriptorProtoLabel" = betterproto.enum_field(4)
    # If type_name is set, this need not be set.  If both this and type_name are
    # set, this must be one of TYPE_ENUM, TYPE_MESSAGE or TYPE_GROUP.
    type: "FieldDescriptorProtoType" = betterproto.enum_field(5)
    # For message and enum types, this is the name of the type.  If the name
    # starts with a '.', it is fully-qualified.  Otherwise, C++-like scoping
    # rules are used to find the type (i.e. first the nested types within this
    # message are searched, then within the parent, on up to the root namespace).
    type_name: str = betterproto.string_field(6)
    # For extensions, this is the name of the type being extended.  It is
    # resolved in the same manner as type_name.
    extendee: str = betterproto.string_field(2)
    # For numeric types, contains the original text representation of the value.
    # For booleans, "true" or "false". For strings, contains the default text
    # contents (not escaped in any way). For bytes, contains the C escaped value.
    # All bytes >= 128 are escaped. TODO(kenton):  Base-64 encode?
    default_value: str = betterproto.string_field(7)
    # If set, gives the index of a oneof in the containing type's oneof_decl
    # list.  This field is a member of that oneof.
    oneof_index: int = betterproto.int32_field(9)
    # JSON name of this field. The value is set by protocol compiler. If the user
    # has set a "json_name" option on this field, that option's value will be
    # used. Otherwise, it's deduced from the field's name by converting it to
    # camelCase.
    json_name: str = betterproto.string_field(10)
    options: "FieldOptions" = betterproto.message_field(8)
    # If true, this is a proto3 "optional". When a proto3 field is optional, it
    # tracks presence regardless of field type. When proto3_optional is true,
    # this field must be belong to a oneof to signal to old proto3 clients that
    # presence is tracked for this field. This oneof is known as a "synthetic"
    # oneof, and this field must be its sole member (each proto3 optional field
    # gets its own synthetic oneof). Synthetic oneofs exist in the descriptor
    # only, and do not generate any API. Synthetic oneofs must be ordered after
    # all "real" oneofs. For message fields, proto3_optional doesn't create any
    # semantic change, since non-repeated message fields always track presence.
    # However it still indicates the semantic detail of whether the user wrote
    # "optional" or not. This can be useful for round-tripping the .proto file.
    # For consistency we give message fields a synthetic oneof also, even though
    # it is not required to track presence. This is especially important because
    # the parser can't tell if a field is a message or an enum, so it must always
    # create a synthetic oneof. Proto2 optional fields do not set this flag,
    # because they already indicate optional with `LABEL_OPTIONAL`.
    proto3_optional: bool = betterproto.bool_field(17)


@dataclass(eq=False, repr=False)
class OneofDescriptorProto(betterproto.Message):
    """Describes a oneof."""

    name: str = betterproto.string_field(1)
    options: "OneofOptions" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class EnumDescriptorProto(betterproto.Message):
    """Describes an enum type."""

    name: str = betterproto.string_field(1)
    value: List["EnumValueDescriptorProto"] = betterproto.message_field(2)
    options: "EnumOptions" = betterproto.message_field(3)
    # Range of reserved numeric values. Reserved numeric values may not be used
    # by enum values in the same enum declaration. Reserved ranges may not
    # overlap.
    reserved_range: List[
        "EnumDescriptorProtoEnumReservedRange"
    ] = betterproto.message_field(4)
    # Reserved enum value names, which may not be reused. A given name may only
    # be reserved once.
    reserved_name: List[str] = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class EnumDescriptorProtoEnumReservedRange(betterproto.Message):
    """
    Range of reserved numeric values. Reserved values may not be used by
    entries in the same enum. Reserved ranges may not overlap. Note that this
    is distinct from DescriptorProto.ReservedRange in that it is inclusive such
    that it can appropriately represent the entire int32 domain.
    """

    start: int = betterproto.int32_field(1)
    end: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class EnumValueDescriptorProto(betterproto.Message):
    """Describes a value within an enum."""

    name: str = betterproto.string_field(1)
    number: int = betterproto.int32_field(2)
    options: "EnumValueOptions" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class ServiceDescriptorProto(betterproto.Message):
    """Describes a service."""

    name: str = betterproto.string_field(1)
    method: List["MethodDescriptorProto"] = betterproto.message_field(2)
    options: "ServiceOptions" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MethodDescriptorProto(betterproto.Message):
    """Describes a method of a service."""

    name: str = betterproto.string_field(1)
    # Input and output type names.  These are resolved in the same way as
    # FieldDescriptorProto.type_name, but must refer to a message type.
    input_type: str = betterproto.string_field(2)
    output_type: str = betterproto.string_field(3)
    options: "MethodOptions" = betterproto.message_field(4)
    # Identifies if client streams multiple client messages
    client_streaming: bool = betterproto.bool_field(5)
    # Identifies if server streams multiple server messages
    server_streaming: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class FileOptions(betterproto.Message):
    # Sets the Java package where classes generated from this .proto will be
    # placed.  By default, the proto package is used, but this is often
    # inappropriate because proto packages do not normally start with backwards
    # domain names.
    java_package: str = betterproto.string_field(1)
    # Controls the name of the wrapper Java class generated for the .proto file.
    # That class will always contain the .proto file's getDescriptor() method as
    # well as any top-level extensions defined in the .proto file. If
    # java_multiple_files is disabled, then all the other classes from the .proto
    # file will be nested inside the single wrapper outer class.
    java_outer_classname: str = betterproto.string_field(8)
    # If enabled, then the Java code generator will generate a separate .java
    # file for each top-level message, enum, and service defined in the .proto
    # file.  Thus, these types will *not* be nested inside the wrapper class
    # named by java_outer_classname.  However, the wrapper class will still be
    # generated to contain the file's getDescriptor() method as well as any top-
    # level extensions defined in the file.
    java_multiple_files: bool = betterproto.bool_field(10)
    # This option does nothing.
    java_generate_equals_and_hash: bool = betterproto.bool_field(20)
    # If set true, then the Java2 code generator will generate code that throws
    # an exception whenever an attempt is made to assign a non-UTF-8 byte
    # sequence to a string field. Message reflection will do the same. However,
    # an extension field still accepts non-UTF-8 byte sequences. This option has
    # no effect on when used with the lite runtime.
    java_string_check_utf8: bool = betterproto.bool_field(27)
    optimize_for: "FileOptionsOptimizeMode" = betterproto.enum_field(9)
    # Sets the Go package where structs generated from this .proto will be
    # placed. If omitted, the Go package will be derived from the following:   -
    # The basename of the package import path, if provided.   - Otherwise, the
    # package statement in the .proto file, if present.   - Otherwise, the
    # basename of the .proto file, without extension.
    go_package: str = betterproto.string_field(11)
    # Should generic services be generated in each language?  "Generic" services
    # are not specific to any particular RPC system.  They are generated by the
    # main code generators in each language (without additional plugins). Generic
    # services were the only kind of service generation supported by early
    # versions of google.protobuf. Generic services are now considered deprecated
    # in favor of using plugins that generate code specific to your particular
    # RPC system.  Therefore, these default to false.  Old code which depends on
    # generic services should explicitly set them to true.
    cc_generic_services: bool = betterproto.bool_field(16)
    java_generic_services: bool = betterproto.bool_field(17)
    py_generic_services: bool = betterproto.bool_field(18)
    php_generic_services: bool = betterproto.bool_field(42)
    # Is this file deprecated? Depending on the target platform, this can emit
    # Deprecated annotations for everything in the file, or it will be completely
    # ignored; in the very least, this is a formalization for deprecating files.
    deprecated: bool = betterproto.bool_field(23)
    # Enables the use of arenas for the proto messages in this file. This applies
    # only to generated classes for C++.
    cc_enable_arenas: bool = betterproto.bool_field(31)
    # Sets the objective c class prefix which is prepended to all objective c
    # generated classes from this .proto. There is no default.
    objc_class_prefix: str = betterproto.string_field(36)
    # Namespace for generated classes; defaults to the package.
    csharp_namespace: str = betterproto.string_field(37)
    # By default Swift generators will take the proto package and CamelCase it
    # replacing '.' with underscore and use that to prefix the types/symbols
    # defined. When this options is provided, they will use this value instead to
    # prefix the types/symbols defined.
    swift_prefix: str = betterproto.string_field(39)
    # Sets the php class prefix which is prepended to all php generated classes
    # from this .proto. Default is empty.
    php_class_prefix: str = betterproto.string_field(40)
    # Use this option to change the namespace of php generated classes. Default
    # is empty. When this option is empty, the package name will be used for
    # determining the namespace.
    php_namespace: str = betterproto.string_field(41)
    # Use this option to change the namespace of php generated metadata classes.
    # Default is empty. When this option is empty, the proto file name will be
    # used for determining the namespace.
    php_metadata_namespace: str = betterproto.string_field(44)
    # Use this option to change the package of ruby generated classes. Default is
    # empty. When this option is not set, the package name will be used for
    # determining the ruby package.
    ruby_package: str = betterproto.string_field(45)
    # The parser stores options it doesn't recognize here. See the documentation
    # for the "Options" section above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.java_generate_equals_and_hash:
            warnings.warn(
                "FileOptions.java_generate_equals_and_hash is deprecated",
                DeprecationWarning,
            )


@dataclass(eq=False, repr=False)
class MessageOptions(betterproto.Message):
    # Set true to use the old proto1 MessageSet wire format for extensions. This
    # is provided for backwards-compatibility with the MessageSet wire format.
    # You should not use this for any other reason:  It's less efficient, has
    # fewer features, and is more complicated. The message must be defined
    # exactly as follows:   message Foo {     option message_set_wire_format =
    # true;     extensions 4 to max;   } Note that the message cannot have any
    # defined fields; MessageSets only have extensions. All extensions of your
    # type must be singular messages; e.g. they cannot be int32s, enums, or
    # repeated messages. Because this is an option, the above two restrictions
    # are not enforced by the protocol compiler.
    message_set_wire_format: bool = betterproto.bool_field(1)
    # Disables the generation of the standard "descriptor()" accessor, which can
    # conflict with a field of the same name.  This is meant to make migration
    # from proto1 easier; new code should avoid fields named "descriptor".
    no_standard_descriptor_accessor: bool = betterproto.bool_field(2)
    # Is this message deprecated? Depending on the target platform, this can emit
    # Deprecated annotations for the message, or it will be completely ignored;
    # in the very least, this is a formalization for deprecating messages.
    deprecated: bool = betterproto.bool_field(3)
    # Whether the message is an automatically generated map entry type for the
    # maps field. For maps fields:     map<KeyType, ValueType> map_field = 1; The
    # parsed descriptor looks like:     message MapFieldEntry {         option
    # map_entry = true;         optional KeyType key = 1;         optional
    # ValueType value = 2;     }     repeated MapFieldEntry map_field = 1;
    # Implementations may choose not to generate the map_entry=true message, but
    # use a native map in the target language to hold the keys and values. The
    # reflection APIs in such implementations still need to work as if the field
    # is a repeated message field. NOTE: Do not set the option in .proto files.
    # Always use the maps syntax instead. The option should only be implicitly
    # set by the proto compiler parser.
    map_entry: bool = betterproto.bool_field(7)
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class FieldOptions(betterproto.Message):
    # The ctype option instructs the C++ code generator to use a different
    # representation of the field than it normally would.  See the specific
    # options below.  This option is not yet implemented in the open source
    # release -- sorry, we'll try to include it in a future version!
    ctype: "FieldOptionsCType" = betterproto.enum_field(1)
    # The packed option can be enabled for repeated primitive fields to enable a
    # more efficient representation on the wire. Rather than repeatedly writing
    # the tag and type for each element, the entire array is encoded as a single
    # length-delimited blob. In proto3, only explicit setting it to false will
    # avoid using packed encoding.
    packed: bool = betterproto.bool_field(2)
    # The jstype option determines the JavaScript type used for values of the
    # field.  The option is permitted only for 64 bit integral and fixed types
    # (int64, uint64, sint64, fixed64, sfixed64).  A field with jstype JS_STRING
    # is represented as JavaScript string, which avoids loss of precision that
    # can happen when a large value is converted to a floating point JavaScript.
    # Specifying JS_NUMBER for the jstype causes the generated JavaScript code to
    # use the JavaScript "number" type.  The behavior of the default option
    # JS_NORMAL is implementation dependent. This option is an enum to permit
    # additional types to be added, e.g. goog.math.Integer.
    jstype: "FieldOptionsJsType" = betterproto.enum_field(6)
    # Should this field be parsed lazily?  Lazy applies only to message-type
    # fields.  It means that when the outer message is initially parsed, the
    # inner message's contents will not be parsed but instead stored in encoded
    # form.  The inner message will actually be parsed when it is first accessed.
    # This is only a hint.  Implementations are free to choose whether to use
    # eager or lazy parsing regardless of the value of this option.  However,
    # setting this option true suggests that the protocol author believes that
    # using lazy parsing on this field is worth the additional bookkeeping
    # overhead typically needed to implement it. This option does not affect the
    # public interface of any generated code; all method signatures remain the
    # same.  Furthermore, thread-safety of the interface is not affected by this
    # option; const methods remain safe to call from multiple threads
    # concurrently, while non-const methods continue to require exclusive access.
    # Note that implementations may choose not to check required fields within a
    # lazy sub-message.  That is, calling IsInitialized() on the outer message
    # may return true even if the inner message has missing required fields. This
    # is necessary because otherwise the inner message would have to be parsed in
    # order to perform the check, defeating the purpose of lazy parsing.  An
    # implementation which chooses not to check required fields must be
    # consistent about it.  That is, for any particular sub-message, the
    # implementation must either *always* check its required fields, or *never*
    # check its required fields, regardless of whether or not the message has
    # been parsed.
    lazy: bool = betterproto.bool_field(5)
    # Is this field deprecated? Depending on the target platform, this can emit
    # Deprecated annotations for accessors, or it will be completely ignored; in
    # the very least, this is a formalization for deprecating fields.
    deprecated: bool = betterproto.bool_field(3)
    # For Google-internal migration only. Do not use.
    weak: bool = betterproto.bool_field(10)
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)

    # NOTE: Manually inserted and matched with https://gitlab.muc.blickfeld.com/daedalus/blickfeld-base/-/merge_requests/73
    # Used by Python client library generator to generate simple numpy accessors for binary fields
    blickfeld_base_numpy_dtype: str = betterproto.string_field(52012)


@dataclass(eq=False, repr=False)
class OneofOptions(betterproto.Message):
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class EnumOptions(betterproto.Message):
    # Set this option to true to allow mapping different tag names to the same
    # value.
    allow_alias: bool = betterproto.bool_field(2)
    # Is this enum deprecated? Depending on the target platform, this can emit
    # Deprecated annotations for the enum, or it will be completely ignored; in
    # the very least, this is a formalization for deprecating enums.
    deprecated: bool = betterproto.bool_field(3)
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class EnumValueOptions(betterproto.Message):
    # Is this enum value deprecated? Depending on the target platform, this can
    # emit Deprecated annotations for the enum value, or it will be completely
    # ignored; in the very least, this is a formalization for deprecating enum
    # values.
    deprecated: bool = betterproto.bool_field(1)
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class ServiceOptions(betterproto.Message):
    # Is this service deprecated? Depending on the target platform, this can emit
    # Deprecated annotations for the service, or it will be completely ignored;
    # in the very least, this is a formalization for deprecating services.
    deprecated: bool = betterproto.bool_field(33)
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class MethodOptions(betterproto.Message):
    # Is this method deprecated? Depending on the target platform, this can emit
    # Deprecated annotations for the method, or it will be completely ignored; in
    # the very least, this is a formalization for deprecating methods.
    deprecated: bool = betterproto.bool_field(33)
    idempotency_level: "MethodOptionsIdempotencyLevel" = betterproto.enum_field(34)
    # The parser stores options it doesn't recognize here. See above.
    uninterpreted_option: List["UninterpretedOption"] = betterproto.message_field(999)


@dataclass(eq=False, repr=False)
class UninterpretedOption(betterproto.Message):
    """
    A message representing a option the parser does not recognize. This only
    appears in options protos created by the compiler::Parser class.
    DescriptorPool resolves these when building Descriptor objects. Therefore,
    options protos in descriptor objects (e.g. returned by
    Descriptor::options(), or produced by Descriptor::CopyTo()) will never have
    UninterpretedOptions in them.
    """

    name: List["UninterpretedOptionNamePart"] = betterproto.message_field(2)
    # The value of the uninterpreted option, in whatever type the tokenizer
    # identified it as during parsing. Exactly one of these should be set.
    identifier_value: str = betterproto.string_field(3)
    positive_int_value: int = betterproto.uint64_field(4)
    negative_int_value: int = betterproto.int64_field(5)
    double_value: float = betterproto.double_field(6)
    string_value: bytes = betterproto.bytes_field(7)
    aggregate_value: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class UninterpretedOptionNamePart(betterproto.Message):
    """
    The name of the uninterpreted option.  Each string represents a segment in
    a dot-separated name.  is_extension is true iff a segment represents an
    extension (denoted with parentheses in options specs in .proto files).
    E.g.,{ ["foo", false], ["bar.baz", true], ["qux", false] } represents
    "foo.(bar.baz).qux".
    """

    name_part: str = betterproto.string_field(1)
    is_extension: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class SourceCodeInfo(betterproto.Message):
    """
    Encapsulates information about the original source file from which a
    FileDescriptorProto was generated.
    """

    # A Location identifies a piece of source code in a .proto file which
    # corresponds to a particular definition.  This information is intended to be
    # useful to IDEs, code indexers, documentation generators, and similar tools.
    # For example, say we have a file like:   message Foo {     optional string
    # foo = 1;   } Let's look at just the field definition:   optional string foo
    # = 1;   ^       ^^     ^^  ^  ^^^   a       bc     de  f  ghi We have the
    # following locations:   span   path               represents   [a,i)  [ 4,
    # 0, 2, 0 ]     The whole field definition.   [a,b)  [ 4, 0, 2, 0, 4 ]  The
    # label (optional).   [c,d)  [ 4, 0, 2, 0, 5 ]  The type (string).   [e,f)  [
    # 4, 0, 2, 0, 1 ]  The name (foo).   [g,h)  [ 4, 0, 2, 0, 3 ]  The number
    # (1). Notes: - A location may refer to a repeated field itself (i.e. not to
    # any   particular index within it).  This is used whenever a set of elements
    # are   logically enclosed in a single code segment.  For example, an entire
    # extend block (possibly containing multiple extension definitions) will
    # have an outer location whose path refers to the "extensions" repeated
    # field without an index. - Multiple locations may have the same path.  This
    # happens when a single   logical declaration is spread out across multiple
    # places.  The most   obvious example is the "extend" block again -- there
    # may be multiple   extend blocks in the same scope, each of which will have
    # the same path. - A location's span is not always a subset of its parent's
    # span.  For   example, the "extendee" of an extension declaration appears at
    # the   beginning of the "extend" block and is shared by all extensions
    # within   the block. - Just because a location's span is a subset of some
    # other location's span   does not mean that it is a descendant.  For
    # example, a "group" defines   both a type and a field in a single
    # declaration.  Thus, the locations   corresponding to the type and field and
    # their components will overlap. - Code which tries to interpret locations
    # should probably be designed to   ignore those that it doesn't understand,
    # as more types of locations could   be recorded in the future.
    location: List["SourceCodeInfoLocation"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SourceCodeInfoLocation(betterproto.Message):
    # Identifies which part of the FileDescriptorProto was defined at this
    # location. Each element is a field number or an index.  They form a path
    # from the root FileDescriptorProto to the place where the definition.  For
    # example, this path:   [ 4, 3, 2, 7, 1 ] refers to:   file.message_type(3)
    # // 4, 3       .field(7)         // 2, 7       .name()           // 1 This
    # is because FileDescriptorProto.message_type has field number 4:   repeated
    # DescriptorProto message_type = 4; and DescriptorProto.field has field
    # number 2:   repeated FieldDescriptorProto field = 2; and
    # FieldDescriptorProto.name has field number 1:   optional string name = 1;
    # Thus, the above path gives the location of a field name.  If we removed the
    # last element:   [ 4, 3, 2, 7 ] this path refers to the whole field
    # declaration (from the beginning of the label to the terminating semicolon).
    path: List[int] = betterproto.int32_field(1)
    # Always has exactly three or four elements: start line, start column, end
    # line (optional, otherwise assumed same as start line), end column. These
    # are packed into a single field for efficiency.  Note that line and column
    # numbers are zero-based -- typically you will want to add 1 to each before
    # displaying to a user.
    span: List[int] = betterproto.int32_field(2)
    # If this SourceCodeInfo represents a complete declaration, these are any
    # comments appearing before and after the declaration which appear to be
    # attached to the declaration. A series of line comments appearing on
    # consecutive lines, with no other tokens appearing on those lines, will be
    # treated as a single comment. leading_detached_comments will keep paragraphs
    # of comments that appear before (but not connected to) the current element.
    # Each paragraph, separated by empty lines, will be one comment element in
    # the repeated field. Only the comment content is provided; comment markers
    # (e.g. //) are stripped out.  For block comments, leading whitespace and an
    # asterisk will be stripped from the beginning of each line other than the
    # first. Newlines are included in the output. Examples:   optional int32 foo
    # = 1;  // Comment attached to foo.   // Comment attached to bar.   optional
    # int32 bar = 2;   optional string baz = 3;   // Comment attached to baz.
    # // Another line attached to baz.   // Comment attached to qux.   //   //
    # Another line attached to qux.   optional double qux = 4;   // Detached
    # comment for corge. This is not leading or trailing comments   // to qux or
    # corge because there are blank lines separating it from   // both.   //
    # Detached comment for corge paragraph 2.   optional string corge = 5;   /*
    # Block comment attached    * to corge.  Leading asterisks    * will be
    # removed. */   /* Block comment attached to    * grault. */   optional int32
    # grault = 6;   // ignored detached comments.
    leading_comments: str = betterproto.string_field(3)
    trailing_comments: str = betterproto.string_field(4)
    leading_detached_comments: List[str] = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class GeneratedCodeInfo(betterproto.Message):
    """
    Describes the relationship between generated code and its original source
    file. A GeneratedCodeInfo message is associated with only one generated
    source file, but may contain references to different source .proto files.
    """

    # An Annotation connects some span of text in generated code to an element of
    # its generating .proto file.
    annotation: List["GeneratedCodeInfoAnnotation"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class GeneratedCodeInfoAnnotation(betterproto.Message):
    # Identifies the element in the original source .proto file. This field is
    # formatted the same as SourceCodeInfo.Location.path.
    path: List[int] = betterproto.int32_field(1)
    # Identifies the filesystem path to the original source .proto.
    source_file: str = betterproto.string_field(2)
    # Identifies the starting offset in bytes in the generated code that relates
    # to the identified object.
    begin: int = betterproto.int32_field(3)
    # Identifies the ending offset in bytes in the generated code that relates to
    # the identified offset. The end offset should be one past the last relevant
    # byte (so the length of the text = end - begin).
    end: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class Duration(betterproto.Message):
    """
    A Duration represents a signed, fixed-length span of time represented as a
    count of seconds and fractions of seconds at nanosecond resolution. It is
    independent of any calendar and concepts like "day" or "month". It is
    related to Timestamp in that the difference between two Timestamp values is
    a Duration and it can be added or subtracted from a Timestamp. Range is
    approximately +-10,000 years. # Examples Example 1: Compute Duration from
    two Timestamps in pseudo code.     Timestamp start = ...;     Timestamp end
    = ...;     Duration duration = ...;     duration.seconds = end.seconds -
    start.seconds;     duration.nanos = end.nanos - start.nanos;     if
    (duration.seconds < 0 && duration.nanos > 0) {       duration.seconds += 1;
    duration.nanos -= 1000000000;     } else if (duration.seconds > 0 &&
    duration.nanos < 0) {       duration.seconds -= 1;       duration.nanos +=
    1000000000;     } Example 2: Compute Timestamp from Timestamp + Duration in
    pseudo code.     Timestamp start = ...;     Duration duration = ...;
    Timestamp end = ...;     end.seconds = start.seconds + duration.seconds;
    end.nanos = start.nanos + duration.nanos;     if (end.nanos < 0) {
    end.seconds -= 1;       end.nanos += 1000000000;     } else if (end.nanos
    >= 1000000000) {       end.seconds += 1;       end.nanos -= 1000000000;
    } Example 3: Compute Duration from datetime.timedelta in Python.     td =
    datetime.timedelta(days=3, minutes=10)     duration = Duration()
    duration.FromTimedelta(td) # JSON Mapping In JSON format, the Duration type
    is encoded as a string rather than an object, where the string ends in the
    suffix "s" (indicating seconds) and is preceded by the number of seconds,
    with nanoseconds expressed as fractional seconds. For example, 3 seconds
    with 0 nanoseconds should be encoded in JSON format as "3s", while 3
    seconds and 1 nanosecond should be expressed in JSON format as
    "3.000000001s", and 3 seconds and 1 microsecond should be expressed in JSON
    format as "3.000001s".
    """

    # Signed seconds of the span of time. Must be from -315,576,000,000 to
    # +315,576,000,000 inclusive. Note: these bounds are computed from: 60
    # sec/min * 60 min/hr * 24 hr/day * 365.25 days/year * 10000 years
    seconds: int = betterproto.int64_field(1)
    # Signed fractions of a second at nanosecond resolution of the span of time.
    # Durations less than one second are represented with a 0 `seconds` field and
    # a positive or negative `nanos` field. For durations of one second or more,
    # a non-zero value for the `nanos` field must be of the same sign as the
    # `seconds` field. Must be from -999,999,999 to +999,999,999 inclusive.
    nanos: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class Empty(betterproto.Message):
    """
    A generic empty message that you can re-use to avoid defining duplicated
    empty messages in your APIs. A typical example is to use it as the request
    or the response type of an API method. For instance:     service Foo {
    rpc Bar(google.protobuf.Empty) returns (google.protobuf.Empty);     } The
    JSON representation for `Empty` is empty JSON object `{}`.
    """

    pass


@dataclass(eq=False, repr=False)
class FieldMask(betterproto.Message):
    """
    `FieldMask` represents a set of symbolic field paths, for example:
    paths: "f.a"     paths: "f.b.d" Here `f` represents a field in some root
    message, `a` and `b` fields in the message found in `f`, and `d` a field
    found in the message in `f.b`. Field masks are used to specify a subset of
    fields that should be returned by a get operation or modified by an update
    operation. Field masks also have a custom JSON encoding (see below). #
    Field Masks in Projections When used in the context of a projection, a
    response message or sub-message is filtered by the API to only contain
    those fields as specified in the mask. For example, if the mask in the
    previous example is applied to a response message as follows:     f {
    a : 22       b {         d : 1         x : 2       }       y : 13     }
    z: 8 The result will not contain specific values for fields x,y and z
    (their value will be set to the default, and omitted in proto text output):
    f {       a : 22       b {         d : 1       }     } A repeated field is
    not allowed except at the last position of a paths string. If a FieldMask
    object is not present in a get operation, the operation applies to all
    fields (as if a FieldMask of all fields had been specified). Note that a
    field mask does not necessarily apply to the top-level response message. In
    case of a REST get operation, the field mask applies directly to the
    response, but in case of a REST list operation, the mask instead applies to
    each individual message in the returned resource list. In case of a REST
    custom method, other definitions may be used. Where the mask applies will
    be clearly documented together with its declaration in the API.  In any
    case, the effect on the returned resource/resources is required behavior
    for APIs. # Field Masks in Update Operations A field mask in update
    operations specifies which fields of the targeted resource are going to be
    updated. The API is required to only change the values of the fields as
    specified in the mask and leave the others untouched. If a resource is
    passed in to describe the updated values, the API ignores the values of all
    fields not covered by the mask. If a repeated field is specified for an
    update operation, new values will be appended to the existing repeated
    field in the target resource. Note that a repeated field is only allowed in
    the last position of a `paths` string. If a sub-message is specified in the
    last position of the field mask for an update operation, then new value
    will be merged into the existing sub-message in the target resource. For
    example, given the target message:     f {       b {         d: 1
    x: 2       }       c: [1]     } And an update message:     f {       b {
    d: 10       }       c: [2]     } then if the field mask is:  paths: ["f.b",
    "f.c"] then the result will be:     f {       b {         d: 10         x:
    2       }       c: [1, 2]     } An implementation may provide options to
    override this default behavior for repeated and message fields. In order to
    reset a field's value to the default, the field must be in the mask and set
    to the default value in the provided resource. Hence, in order to reset all
    fields of a resource, provide a default instance of the resource and set
    all fields in the mask, or do not provide a mask as described below. If a
    field mask is not present on update, the operation applies to all fields
    (as if a field mask of all fields has been specified). Note that in the
    presence of schema evolution, this may mean that fields the client does not
    know and has therefore not filled into the request will be reset to their
    default. If this is unwanted behavior, a specific service may require a
    client to always specify a field mask, producing an error if not. As with
    get operations, the location of the resource which describes the updated
    values in the request message depends on the operation kind. In any case,
    the effect of the field mask is required to be honored by the API. ##
    Considerations for HTTP REST The HTTP kind of an update operation which
    uses a field mask must be set to PATCH instead of PUT in order to satisfy
    HTTP semantics (PUT must only be used for full updates). # JSON Encoding of
    Field Masks In JSON, a field mask is encoded as a single string where paths
    are separated by a comma. Fields name in each path are converted to/from
    lower-camel naming conventions. As an example, consider the following
    message declarations:     message Profile {       User user = 1;
    Photo photo = 2;     }     message User {       string display_name = 1;
    string address = 2;     } In proto a field mask for `Profile` may look as
    such:     mask {       paths: "user.display_name"       paths: "photo"
    } In JSON, the same mask is represented as below:     {       mask:
    "user.displayName,photo"     } # Field Masks and Oneof Fields Field masks
    treat fields in oneofs just as regular fields. Consider the following
    message:     message SampleMessage {       oneof test_oneof {
    string name = 4;         SubMessage sub_message = 9;       }     } The
    field mask can be:     mask {       paths: "name"     } Or:     mask {
    paths: "sub_message"     } Note that oneof type names ("test_oneof" in this
    case) cannot be used in paths. ## Field Mask Verification The
    implementation of any API method which has a FieldMask type field in the
    request should verify the included field paths, and return an
    `INVALID_ARGUMENT` error if any path is unmappable.
    """

    # The set of field mask paths.
    paths: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class Struct(betterproto.Message):
    """
    `Struct` represents a structured data value, consisting of fields which map
    to dynamically typed values. In some languages, `Struct` might be supported
    by a native representation. For example, in scripting languages like JS a
    struct is represented as an object. The details of that representation are
    described together with the proto support for the language. The JSON
    representation for `Struct` is JSON object.
    """

    # Unordered map of dynamically typed values.
    fields: Dict[str, "Value"] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )


@dataclass(eq=False, repr=False)
class Value(betterproto.Message):
    """
    `Value` represents a dynamically typed value which can be either null, a
    number, a string, a boolean, a recursive struct value, or a list of values.
    A producer of value is expected to set one of that variants, absence of any
    variant indicates an error. The JSON representation for `Value` is JSON
    value.
    """

    # Represents a null value.
    null_value: "NullValue" = betterproto.enum_field(1, group="kind")
    # Represents a double value.
    number_value: float = betterproto.double_field(2, group="kind")
    # Represents a string value.
    string_value: str = betterproto.string_field(3, group="kind")
    # Represents a boolean value.
    bool_value: bool = betterproto.bool_field(4, group="kind")
    # Represents a structured value.
    struct_value: "Struct" = betterproto.message_field(5, group="kind")
    # Represents a repeated `Value`.
    list_value: "ListValue" = betterproto.message_field(6, group="kind")


@dataclass(eq=False, repr=False)
class ListValue(betterproto.Message):
    """
    `ListValue` is a wrapper around a repeated field of values. The JSON
    representation for `ListValue` is JSON array.
    """

    # Repeated field of dynamically typed values.
    values: List["Value"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Timestamp(betterproto.Message):
    """
    A Timestamp represents a point in time independent of any time zone or
    local calendar, encoded as a count of seconds and fractions of seconds at
    nanosecond resolution. The count is relative to an epoch at UTC midnight on
    January 1, 1970, in the proleptic Gregorian calendar which extends the
    Gregorian calendar backwards to year one. All minutes are 60 seconds long.
    Leap seconds are "smeared" so that no leap second table is needed for
    interpretation, using a [24-hour linear
    smear](https://developers.google.com/time/smear). The range is from
    0001-01-01T00:00:00Z to 9999-12-31T23:59:59.999999999Z. By restricting to
    that range, we ensure that we can convert to and from [RFC
    3339](https://www.ietf.org/rfc/rfc3339.txt) date strings. # Examples
    Example 1: Compute Timestamp from POSIX `time()`.     Timestamp timestamp;
    timestamp.set_seconds(time(NULL));     timestamp.set_nanos(0); Example 2:
    Compute Timestamp from POSIX `gettimeofday()`.     struct timeval tv;
    gettimeofday(&tv, NULL);     Timestamp timestamp;
    timestamp.set_seconds(tv.tv_sec);     timestamp.set_nanos(tv.tv_usec *
    1000); Example 3: Compute Timestamp from Win32 `GetSystemTimeAsFileTime()`.
    FILETIME ft;     GetSystemTimeAsFileTime(&ft);     UINT64 ticks =
    (((UINT64)ft.dwHighDateTime) << 32) | ft.dwLowDateTime;     // A Windows
    tick is 100 nanoseconds. Windows epoch 1601-01-01T00:00:00Z     // is
    11644473600 seconds before Unix epoch 1970-01-01T00:00:00Z.     Timestamp
    timestamp;     timestamp.set_seconds((INT64) ((ticks / 10000000) -
    11644473600LL));     timestamp.set_nanos((INT32) ((ticks % 10000000) *
    100)); Example 4: Compute Timestamp from Java `System.currentTimeMillis()`.
    long millis = System.currentTimeMillis();     Timestamp timestamp =
    Timestamp.newBuilder().setSeconds(millis / 1000)         .setNanos((int)
    ((millis % 1000) * 1000000)).build(); Example 5: Compute Timestamp from
    Java `Instant.now()`.     Instant now = Instant.now();     Timestamp
    timestamp =         Timestamp.newBuilder().setSeconds(now.getEpochSecond())
    .setNanos(now.getNano()).build(); Example 6: Compute Timestamp from current
    time in Python.     timestamp = Timestamp()     timestamp.GetCurrentTime()
    # JSON Mapping In JSON format, the Timestamp type is encoded as a string in
    the [RFC 3339](https://www.ietf.org/rfc/rfc3339.txt) format. That is, the
    format is "{year}-{month}-{day}T{hour}:{min}:{sec}[.{frac_sec}]Z" where
    {year} is always expressed using four digits while {month}, {day}, {hour},
    {min}, and {sec} are zero-padded to two digits each. The fractional
    seconds, which can go up to 9 digits (i.e. up to 1 nanosecond resolution),
    are optional. The "Z" suffix indicates the timezone ("UTC"); the timezone
    is required. A proto3 JSON serializer should always use UTC (as indicated
    by "Z") when printing the Timestamp type and a proto3 JSON parser should be
    able to accept both UTC and other timezones (as indicated by an offset).
    For example, "2017-01-15T01:30:15.01Z" encodes 15.01 seconds past 01:30 UTC
    on January 15, 2017. In JavaScript, one can convert a Date object to this
    format using the standard [toISOString()](https://developer.mozilla.org/en-
    US/docs/Web/JavaScript/Reference/Global_Objects/Date/toISOString) method.
    In Python, a standard `datetime.datetime` object can be converted to this
    format using
    [`strftime`](https://docs.python.org/2/library/time.html#time.strftime)
    with the time format spec '%Y-%m-%dT%H:%M:%S.%fZ'. Likewise, in Java, one
    can use the Joda Time's [`ISODateTimeFormat.dateTime()`](
    http://www.joda.org/joda-
    time/apidocs/org/joda/time/format/ISODateTimeFormat.html#dateTime%2D%2D )
    to obtain a formatter capable of generating timestamps in this format.
    """

    # Represents seconds of UTC time since Unix epoch 1970-01-01T00:00:00Z. Must
    # be from 0001-01-01T00:00:00Z to 9999-12-31T23:59:59Z inclusive.
    seconds: int = betterproto.int64_field(1)
    # Non-negative fractions of a second at nanosecond resolution. Negative
    # second values with fractions must still have non-negative nanos values that
    # count forward in time. Must be from 0 to 999,999,999 inclusive.
    nanos: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class DoubleValue(betterproto.Message):
    """
    Wrapper message for `double`. The JSON representation for `DoubleValue` is
    JSON number.
    """

    # The double value.
    value: float = betterproto.double_field(1)


@dataclass(eq=False, repr=False)
class FloatValue(betterproto.Message):
    """
    Wrapper message for `float`. The JSON representation for `FloatValue` is
    JSON number.
    """

    # The float value.
    value: float = betterproto.float_field(1)


@dataclass(eq=False, repr=False)
class Int64Value(betterproto.Message):
    """
    Wrapper message for `int64`. The JSON representation for `Int64Value` is
    JSON string.
    """

    # The int64 value.
    value: int = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class UInt64Value(betterproto.Message):
    """
    Wrapper message for `uint64`. The JSON representation for `UInt64Value` is
    JSON string.
    """

    # The uint64 value.
    value: int = betterproto.uint64_field(1)


@dataclass(eq=False, repr=False)
class Int32Value(betterproto.Message):
    """
    Wrapper message for `int32`. The JSON representation for `Int32Value` is
    JSON number.
    """

    # The int32 value.
    value: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class UInt32Value(betterproto.Message):
    """
    Wrapper message for `uint32`. The JSON representation for `UInt32Value` is
    JSON number.
    """

    # The uint32 value.
    value: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class BoolValue(betterproto.Message):
    """
    Wrapper message for `bool`. The JSON representation for `BoolValue` is JSON
    `true` and `false`.
    """

    # The bool value.
    value: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class StringValue(betterproto.Message):
    """
    Wrapper message for `string`. The JSON representation for `StringValue` is
    JSON string.
    """

    # The string value.
    value: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class BytesValue(betterproto.Message):
    """
    Wrapper message for `bytes`. The JSON representation for `BytesValue` is
    JSON string.
    """

    # The bytes value.
    value: bytes = betterproto.bytes_field(1)





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/laser/data/health.proto, blickfeld/laser/data/mode.proto
# plugin: python-betterproto
from dataclasses import dataclass

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ...base import data as __base_data__


class Mode(betterproto.Enum):
    """Operation mode of the laser module"""

    MODE_UNSPECIFIED = 0
    """Unused"""

    MODE_IDLE = 1
    """Nothing is happening, waiting to start."""

    MODE_RAMP_UP = 2
    """
    There were no triggers for a longer time, hence we have to ramp up. Only
    90% of pulse energy is used during triggering.
    """

    MODE_ACTIVE = 3
    """
    Normal mode with triggering happening frequently and laser is triggered
    with max pulse energy according to mode.
    """

    MODE_ERROR = 4
    """
    Something went wrong, laser module is in error state. Please check the
    details in the health status of laser.
    """


@dataclass(eq=False, repr=False)
class Health(betterproto.Message):
    state: "__base_data__.HealthState" = betterproto.enum_field(1)
    """High-level state of module"""

    state_reason: str = betterproto.string_field(2)
    """Reason for given state. Is not set if state is OK."""

    reference_temperature: float = betterproto.float_field(6)
    """Current temperature measured with dedicated temperature IC on PCB"""

    mcu_temperature: float = betterproto.float_field(7)
    """Current core temperature of microcontroller"""

    mode: "Mode" = betterproto.enum_field(8)
    """Current Laser operating mode, it has impact on pulse energy"""

    trigger_frequency: float = betterproto.double_field(9)
    """
    The frequency of the laser trigger with an error of +/-pre-scaler value
    """





# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: blickfeld/flow/services/credentials.proto, blickfeld/flow/services/flow.proto, blickfeld/flow/services/settings.proto
# plugin: python-betterproto
import asyncio
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    AsyncIterator,
    Dict,
    Iterator,
    List,
    Optional,
)

import blickfeld_qb2.betterproto as betterproto
import blickfeld_qb2.betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from blickfeld_qb2.betterproto.grpc.grpclib_server import ServiceBase

from ...config import data as __config_data__
from .. import config as _config__


if TYPE_CHECKING:
    import grpclib.server
    from blickfeld_qb2.betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class CredentialsSetRequest(betterproto.Message):
    """Credentials set request"""

    credentials: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """
    Credentials which should be stored   Generic JSON format which is defined
    by NodeRed.
    """


@dataclass(eq=False, repr=False)
class CredentialsGetResponse(betterproto.Message):
    """Credentails get response"""

    credentials: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """
    Current credentials   Generic JSON format which is defined by NodeRed.
    """


@dataclass(eq=False, repr=False)
class FlowGetRequest(betterproto.Message):
    """Flow get request"""

    id: str = betterproto.string_field(1)
    """Flow id"""

    as_node_red_json: bool = betterproto.bool_field(2)
    """
    If true, the request returns the NodeRed JSON format which can be directly
    imported.
    """


@dataclass(eq=False, repr=False)
class FlowGetResponse(betterproto.Message):
    """Flow get response"""

    flow: "_config__.Flow" = betterproto.message_field(1, group="kind")
    """Blickfeld Flow config with well-defined high-level attributes"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(2, group="kind")
    """NodeRed JSON format which can be directly imported"""


@dataclass(eq=False, repr=False)
class FlowListResponse(betterproto.Message):
    """Flow list response"""

    ids: List[str] = betterproto.string_field(1)
    """List of Blickfeld Flow identifiers."""


@dataclass(eq=False, repr=False)
class FlowStoreRequest(betterproto.Message):
    """Flow store request"""

    id: str = betterproto.string_field(1)
    """
    ID of the flow   NOTE: For node_red_json, this has to match the ID in the
    NodeRED JSON format.
    """

    flow: "_config__.Flow" = betterproto.message_field(2, group="kind")
    """Blickfeld Flow config with well-defined high-level attributes"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(3, group="kind")
    """
    NodeRed JSON format which can be directly exported from NodeRED Admin UI
    """


@dataclass(eq=False, repr=False)
class FlowDeleteRequest(betterproto.Message):
    """Flow delete request"""

    id: str = betterproto.string_field(1)
    """Id of Blickfeld Flow"""


@dataclass(eq=False, repr=False)
class FlowWatchRequest(betterproto.Message):
    """Flow watch request"""

    as_node_red_json: bool = betterproto.bool_field(1)
    """
    If true, the request returns the NodeRed JSON format which can be directly
    imported.
    """


@dataclass(eq=False, repr=False)
class FlowWatchResponse(betterproto.Message):
    """Per change, a response is emitted"""

    id: str = betterproto.string_field(1)
    """Id of Blickfeld Flow"""

    flow: "_config__.Flow" = betterproto.message_field(2, group="kind")
    """Blickfeld Flow config with well-defined high-level attributes"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(3, group="kind")
    """
    NodeRed JSON format which can be directly exported from NodeRED Admin UI
    """

    type: "__config_data__.WatchEventType" = betterproto.enum_field(4)
    """Type of change"""


@dataclass(eq=False, repr=False)
class FlowStoreGlobalNodesRequest(betterproto.Message):
    """Flow store global nodes request"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(1)
    """NodeRed JSON format of all global configuration nodes"""


@dataclass(eq=False, repr=False)
class FlowGetGlobalNodesResponse(betterproto.Message):
    """Flow get global nodes response"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(1)
    """NodeRed JSON format of all global configuration nodes"""


@dataclass(eq=False, repr=False)
class FlowWatchGlobalNodesResponse(betterproto.Message):
    """Flow watch global nodes response"""

    node_red_json: "_config__.NodeRedJson" = betterproto.message_field(1)
    """NodeRed JSON format of all global configuration nodes"""


@dataclass(eq=False, repr=False)
class SettingsSetRequest(betterproto.Message):
    """Settings set request"""

    settings: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """
    Settings which should be stored   Generic JSON format which is defined by
    NodeRed.
    """


@dataclass(eq=False, repr=False)
class SettingsGetResponse(betterproto.Message):
    """Settings get request"""

    settings: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    """Current settings   Generic JSON format which is defined by NodeRed."""


class Credentials(betterproto.ServiceStub):
    """
    Service to store the NodeRed credentials.   It is used to implement the
    custom storage API: https://nodered.org/docs/api/storage/
    """

    async def async_set(
        self,
        *,
        credentials: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set credentials   Used by https://nodered.org/docs/api/storage/methods/
        #storagesavecredentialscredentials
        """

        request = CredentialsSetRequest()
        if credentials is not None:
            request.credentials = credentials

        return await self._unary_unary(
            "/blickfeld.flow.services.Credentials/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        credentials: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set credentials   Used by https://nodered.org/docs/api/storage/methods/
        #storagesavecredentialscredentials
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                credentials=credentials,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "CredentialsGetResponse":
        """
        Get credentials   Used by
        https://nodered.org/docs/api/storage/methods/#storagegetcredentials
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Credentials/Get",
            request,
            CredentialsGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "CredentialsGetResponse":
        """
        Get credentials   Used by
        https://nodered.org/docs/api/storage/methods/#storagegetcredentials
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Flow(betterproto.ServiceStub):
    """
    Service to store and control Blickfeld flows.   It is used to implement the
    custom storage API: https://nodered.org/docs/api/storage/
    """

    async def async_get(
        self,
        *,
        id: str = "",
        as_node_red_json: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "FlowGetResponse":
        """Get a flow by id"""

        request = FlowGetRequest()
        request.id = id
        request.as_node_red_json = as_node_red_json

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Get",
            request,
            FlowGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        *,
        id: str = "",
        as_node_red_json: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "FlowGetResponse":
        """Get a flow by id"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                id=id,
                as_node_red_json=as_node_red_json,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowListResponse":
        """Return list of flow ids"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/List",
            request,
            FlowListResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def list(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowListResponse":
        """Return list of flow ids"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_list(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_store(
        self,
        *,
        id: str = "",
        flow: "_config__.Flow" = None,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store flow   A "Reload" is required to apply the operation."""

        request = FlowStoreRequest()
        request.id = id
        if flow is not None:
            request.flow = flow
        if node_red_json is not None:
            request.node_red_json = node_red_json

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Store",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store(
        self,
        *,
        id: str = "",
        flow: "_config__.Flow" = None,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """Store flow   A "Reload" is required to apply the operation."""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store(
                id=id,
                flow=flow,
                node_red_json=node_red_json,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_delete(
        self,
        *,
        id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Delete stored flow   A "Reload" is required to apply the operation.
        """

        request = FlowDeleteRequest()
        request.id = id

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Delete",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def delete(
        self,
        *,
        id: str = "",
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Delete stored flow   A "Reload" is required to apply the operation.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_delete(
                id=id,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch(
        self,
        *,
        as_node_red_json: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> AsyncIterator["FlowWatchResponse"]:
        """Watch flow changes"""

        request = FlowWatchRequest()
        request.as_node_red_json = as_node_red_json

        async for response in self._unary_stream(
            "/blickfeld.flow.services.Flow/Watch",
            request,
            FlowWatchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ):
            yield response

    def watch(
        self,
        *,
        as_node_red_json: bool = False,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> Iterator["FlowWatchResponse"]:
        """Watch flow changes"""

        loop = asyncio.get_event_loop()
        ait = self.async_watch(
            as_node_red_json=as_node_red_json,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        ).__aiter__()

        async def get_next():
            try:
                obj = await ait.__anext__()
                return False, obj
            except StopAsyncIteration:
                return True, None

        while True:
            done, obj = loop.run_until_complete(get_next())
            if done:
                break
            yield obj

    async def async_store_global_nodes(
        self,
        *,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Store global nodes   A "Reload" is required to apply the operation.
        """

        request = FlowStoreGlobalNodesRequest()
        if node_red_json is not None:
            request.node_red_json = node_red_json

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/StoreGlobalNodes",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def store_global_nodes(
        self,
        *,
        node_red_json: "_config__.NodeRedJson" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Store global nodes   A "Reload" is required to apply the operation.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_store_global_nodes(
                node_red_json=node_red_json,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get_global_nodes(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowGetGlobalNodesResponse":
        """Get global nodes"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/GetGlobalNodes",
            request,
            FlowGetGlobalNodesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get_global_nodes(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowGetGlobalNodesResponse":
        """Get global nodes"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get_global_nodes(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_watch_global_nodes(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowWatchGlobalNodesResponse":
        """Watch global nodes"""

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/WatchGlobalNodes",
            request,
            FlowWatchGlobalNodesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def watch_global_nodes(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "FlowWatchGlobalNodesResponse":
        """Watch global nodes"""

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_watch_global_nodes(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_reload(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Reload node-red   This reloads all nodes from the config and restart
        all nodes.
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Flow/Reload",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def reload(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Reload node-red   This reloads all nodes from the config and restart
        all nodes.
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_reload(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )


class Settings(betterproto.ServiceStub):
    """
    Configuration service to store the NodeRed Settings.   It is used to
    implement the custom storage API: https://nodered.org/docs/api/storage/
    """

    async def async_set(
        self,
        *,
        settings: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set settings   Used by https://nodered.org/docs/api/storage/methods/#st
        oragesavesettingssettings
        """

        request = SettingsSetRequest()
        if settings is not None:
            request.settings = settings

        return await self._unary_unary(
            "/blickfeld.flow.services.Settings/Set",
            request,
            betterproto_lib_google_protobuf.Empty,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def set(
        self,
        *,
        settings: "betterproto_lib_google_protobuf.Struct" = None,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "betterproto_lib_google_protobuf.Empty":
        """
        Set settings   Used by https://nodered.org/docs/api/storage/methods/#st
        oragesavesettingssettings
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_set(
                settings=settings,
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )

    async def async_get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "SettingsGetResponse":
        """
        Get settings   Used by
        https://nodered.org/docs/api/storage/methods/#storagegetsettings
        """

        request = betterproto_lib_google_protobuf.Empty()

        return await self._unary_unary(
            "/blickfeld.flow.services.Settings/Get",
            request,
            SettingsGetResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    def get(
        self,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None,
    ) -> "SettingsGetResponse":
        """
        Get settings   Used by
        https://nodered.org/docs/api/storage/methods/#storagegetsettings
        """

        loop = asyncio.get_event_loop()
        return loop.run_until_complete(
            self.async_get(
                timeout=timeout,
                deadline=deadline,
                metadata=metadata,
            )
        )



import asyncio
import contextlib
import inspect
import logging
import os
from collections.abc import Awaitable, Callable
from types import TracebackType

from grpc import (
    AuthMetadataContext,
    AuthMetadataPlugin,
    AuthMetadataPluginCallback,
    ChannelConnectivity,
    StatusCode,
    composite_channel_credentials,
    metadata_call_credentials,
    ssl_channel_credentials,
)
from grpc.aio import AioRpcError, Channel, Metadata, insecure_channel, secure_channel

from blickfeld.base.grpc.base_error import BaseError

from .constants import OPTIONS
from .device_ca_cert import ca_cert

logger = logging.getLogger("channel")


class AioChannel:
    """
    Channel class is used to securely connect to a Qb2 device.

    The connection is encrypted and authenticated.
    The serial_number can be passed additionally to authenticate a particular device.
    If it is not supplied, the authentication only checks if it is a Blickfeld device.

    ::param fqdn_or_ip: Fully qualified domain name e.g. qb2-xxxxx.blickfeld.com, hostname, or IP address
    :type fqdn_or_ip: str
    param serial_number: Unique serial number assigned to each Qb2 device (corresponding serial number is written on the device label)
    :type serial_number: str
    :param port: Port on which the Qb2 device is reachable (the default is 55551)
    :type port: uint
    :param device_ca_cert: String containing a root certificate for SSL connection
    :type device_ca_cert: str
    :param token: String containing an access token or an awaitable which is called on every method call
    """

    def __init__(
        self,
        fqdn_or_ip: str,
        serial_number: str | None = None,
        port: int | None = None,
        device_ca_cert: str = ca_cert,
        token: str | Callable[["AioChannel"], str] | Awaitable[str] | None = None,
        metadata: Metadata | None = None,
        secure: bool | None = None,
        timeout: int | None = 10,
    ) -> None:
        metadata = metadata or Metadata()

        # Store all properties to allow clone
        self.fqdn_or_ip = fqdn_or_ip
        self.serial_number = serial_number
        self.port = port
        self.device_ca_cert = device_ca_cert
        self.token = token
        self.metadata = metadata
        self.secure = (
            os.getenv("BF_ALLOW_INSECURE_CONNECTIONS") is None or token is not None if secure is None else secure
        )
        self.timeout = timeout

        self.logger = logging.getLogger("Channel")

        # Set name for hostname check to Qb2 device name
        device_name = serial_number + ".qb2.blickfeld.com" if serial_number else None

        this = self

        class TokenPlugin(AuthMetadataPlugin):
            def __call__(self, context: AuthMetadataContext, callback: AuthMetadataPluginCallback) -> None:
                async def inject() -> None:
                    try:
                        try:
                            token = this.token
                            if callable(token):
                                token = token(this)
                            if inspect.isawaitable(token):
                                token = await token

                            if token:
                                metadata.set_all("token", values=[token])

                            callback(metadata=metadata, error=None)
                        except ExceptionGroup as ex:
                            raise ex.exceptions[0]
                    except AioRpcError as ex:
                        callback(metadata=metadata, error=f"{ex.code().name}: {ex.details()}")
                    except Exception as ex:
                        callback(metadata=metadata, error=ex)

                this._task_group.create_task(inject())  # noqa: SLF001

        token_call_credentials = metadata_call_credentials(TokenPlugin())

        # Establish TLS connection to verify connectivity
        if self.secure:
            self.port = port or 55551
            credentials = composite_channel_credentials(
                ssl_channel_credentials(
                    root_certificates=device_ca_cert.encode(), private_key=None, certificate_chain=None
                ),
                token_call_credentials,
            )
            self._channel = secure_channel(
                f"{fqdn_or_ip}:{self.port}",
                credentials,
                ([("grpc.ssl_target_name_override", device_name)] if device_name else []) + OPTIONS,
            )
        else:
            self.logger.info(f"Using insecure connection for {fqdn_or_ip} as BF_ALLOW_INSECURE_CONNECTIONS is set")
            self.port = port or 50051
            self._channel = insecure_channel(f"{fqdn_or_ip}:{self.port}", options=OPTIONS)

    def clone(self, token: str | Callable[["AioChannel"], str] | Awaitable[str] | None = None) -> "AioChannel":
        return AioChannel(
            fqdn_or_ip=self.fqdn_or_ip,
            serial_number=self.serial_number,
            port=self.port,
            device_ca_cert=self.device_ca_cert,
            token=token if token != self.token else self.token,
            metadata=self.metadata,
            secure=self.secure,
            timeout=self.timeout,
        )

    async def __aenter__(self) -> Channel:
        self._task_group = await asyncio.TaskGroup().__aenter__()

        await self._channel.__aenter__()

        try:
            while True:
                last_observed_state = self._channel.get_state(try_to_connect=True)

                if last_observed_state in [
                    ChannelConnectivity.READY,
                    ChannelConnectivity.TRANSIENT_FAILURE,
                ]:
                    break

                try:
                    await asyncio.wait_for(self._channel.wait_for_state_change(last_observed_state), self.timeout)
                except TimeoutError:
                    raise BaseError(
                        StatusCode.UNAVAILABLE,
                        f"Connection to Qb device '{self.fqdn_or_ip}' failed. Deadline exceeded.",
                    )

            if last_observed_state != ChannelConnectivity.READY:
                raise BaseError(
                    StatusCode.UNAVAILABLE, f"Connection to Qb device '{self.fqdn_or_ip}' failed. Network failure."
                )
        except Exception:
            await self.__aexit__(None, None, None)
            raise

        return self._channel

    async def __aexit__(
        self,
        exception_type: type[BaseException] | None,
        exception_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> bool | None:
        with contextlib.suppress(Exception):
            await self._task_group.__aexit__(exception_type, exception_value, traceback)

        await self._channel.close()



from abc import ABC
from collections.abc import AsyncIterable
from typing import (
    Any,
    Callable,
    Dict,
)

import grpclib
import grpclib.server


class ServiceBase(ABC):
    """
    Base class for async gRPC servers.
    """

    async def _call_rpc_handler_server_stream(
        self,
        handler: Callable,
        stream: grpclib.server.Stream,
        request_kwargs: Dict[str, Any],
    ) -> None:

        response_iter = handler(**request_kwargs)
        # check if response is actually an AsyncIterator
        # this might be false if the method just returns without
        # yielding at least once
        # in that case, we just interpret it as an empty iterator
        if isinstance(response_iter, AsyncIterable):
            async for response_message in response_iter:
                await stream.send_message(response_message)
        else:
            response_iter.close()
